from __future__ import print_function

from enum import Enum
from typing import Any

from java.io import Closeable, Flushable, Reader, Writer
from java.lang import Object, String


class JsonReader(Object, Closeable):
    def __init__(self, in_):
        # type: (Reader) -> None
        print(in_)

    def beginArray(self):
        # type: () -> None
        pass

    def beginObject(self):
        # type: () -> None
        pass

    def close(self):
        # type: () -> None
        pass

    def endArray(self):
        # type: () -> None
        pass

    def endObject(self):
        # type: () -> None
        pass

    def getPath(self):
        # type: () -> String
        pass

    def hasNext(self):
        # type: () -> bool
        pass

    def isLenient(self):
        # type: () -> bool
        pass

    def nextBoolean(self):
        # type: () -> bool
        pass

    def nextDouble(self):
        # type: () -> float
        pass

    def nextInt(self):
        # type: () -> int
        pass

    def nextLong(self):
        # type: () -> long
        pass

    def nextName(self):
        # type: () -> String
        pass

    def nextNull(self):
        # type: () -> None
        pass

    def nextString(self):
        # type: () -> String
        pass

    def peek(self):
        # type: () -> JsonToken
        pass

    def setLenient(self, lenient):
        # type: (bool) -> None
        pass

    def skipValue(self):
        # type: () -> None
        pass


class JsonToken(Enum):
    BEGIN_ARRAY = None  # type: JsonToken
    BEGIN_OBJECT = None  # type: JsonToken
    BOOLEAN = None  # type: JsonToken
    END_ARRAY = None  # type: JsonToken
    END_DOCUMENT = None  # type: JsonToken
    END_OBJECT = None  # type: JsonToken
    NAME = None  # type: JsonToken
    NULL = None  # type: JsonToken
    NUMBER = None  # type: JsonToken
    STRING = None  # type: JsonToken


class JsonWriter(Object, Closeable, Flushable):
    def __init__(self, out):
        # type: (Writer) -> None
        print(out)

    def beginArray(self):
        # type: () -> JsonWriter
        pass

    def beginObject(self):
        # type: () -> JsonWriter
        pass

    def close(self):
        # type: () -> None
        pass

    def endArray(self):
        # type: () -> JsonWriter
        pass

    def endObject(self):
        # type: () -> JsonWriter
        pass

    def flush(self):
        # type: () -> None
        pass

    def getSerializeNulls(self):
        # type: () -> bool
        pass

    def isHtmlSafe(self):
        # type: () -> bool
        pass

    def isLenient(self):
        # type: () -> bool
        pass

    def jsonValue(self, value):
        # type: (String) -> JsonWriter
        pass

    def name(self, name):
        # type: (String) -> JsonWriter
        pass

    def setHtmlSafe(self, htmlSafe):
        # type: (bool) -> None
        pass

    def setIndent(self, indent):
        # type: (String) -> None
        pass

    def setLenient(self, lenient):
        # type: (bool) -> None
        pass

    def setSerializeNulls(self, serializeNulls):
        # type: (bool) -> None
        pass

    def value(self, arg):
        # type: (Any) -> JsonWriter
        pass



from __future__ import print_function

__all__ = ["BufferedImage"]

from typing import Any

from java.awt import Image


class BufferedImage(Image):
    def __init__(self, *args):
        # type: (*Any) -> None
        print(args)



from __future__ import print_function

__all__ = ["SimplifiedTagPath"]

from com.inductiveautomation.ignition.common.tags.paths import AbstractTagPath


class SimplifiedTagPath(AbstractTagPath):
    def __init__(self, delegate):
        print(delegate)



"""EAM Functions.

The following functions give you access to view EAM information from the
Gateway.
"""

from __future__ import print_function

__all__ = ["getGroups", "queryAgentHistory", "queryAgentStatus", "runTask"]

from typing import List, Optional

from com.inductiveautomation.ignition.common import BasicDataset
from com.inductiveautomation.ignition.common.messages import UIResponse
from java.lang import String
from java.util import Date, Locale


def getGroups():
    # type: () -> List[String]
    """Returns the names of the defined agent organizational groups in
    the Gateway.

    This function should only be called from the Controller.

    Note:
        If called from an Agent on 8.1.11+, this function will return an
        exception.

    Returns:
        A string list of group names.
    """
    return [""]


def queryAgentHistory(
    groupIds=None,  # type: Optional[List[String]]
    agentIds=None,  # type: Optional[List[String]]
    startDate=None,  # type: Optional[Date]
    endDate=None,  # type: Optional[Date]
    limit=100,  # type: int
):
    # type: (...) -> BasicDataset
    """Returns a list of the most recent agent events.

    This function should only be called from the Controller.

    Note:
        If called from an Agent on 8.1.11+, this function will return an
        exception.

    Args:
        groupIds: A list of groups to restrict the results to. If not
            specified, all groups will be included. Optional.
        agentIds: A list of agent names to restrict the results to. If
            not specified, all agents will be allowed. Optional.
        startDate: The starting time for history events. If null,
            defaults to 8 hours previous to now. Optional.
        endDate: The ending time for the query range. If null, defaults
            to "now". Optional.
        limit: The limit of results to return. Defaults to 100. A value
            of 0 means "no limit". Optional.

    Returns:
        A dataset with columns id, agent_name, agent_role, event_time,
        event_category, event_type, event_source, event_level,
        event_level_int, and message, where each row is a new agent
        event.
    """
    print(groupIds, agentIds, startDate, endDate, limit)
    return BasicDataset()


def queryAgentStatus(
    groupIds=None,  # type: Optional[List[String]]
    agentIds=None,  # type: Optional[List[String]]
    isConnected=True,  # type: bool
):
    # type: (...) -> BasicDataset
    """Returns the current state of the matching agents.

    This function should only be called from the Controller.

    Note:
        If called from an Agent on 8.1.11+, this function will return an
        exception.

    Args:
        groupIds: A list of groups to restrict the results to. If not
            specified, all groups will be included. Optional.
        agentIds: A list of agent names to restrict the results to. If
            not specified, all agents will be allowed. Optional.
        isConnected: If True, only returns agents that are currently
            connected. If False, only agents that are considered down
            will be returned, and if not specified, all agents will be
            returned. Optional.

    Returns:
        A dataset with columns AgentName, NodeRole, AgentGroup,
        LastCommunication, IsConnected, IsRunning, RunningState,
        RunningStateInt, LicenseKey, and Version, where each row is a
        new agent.

        Possible values for RunningState and RunningStateInt are:
        0 = Disconnected, 1 = Running, 2 = Warned, 3 = Errored.
    """
    print(groupIds, agentIds, isConnected)
    return BasicDataset()


def runTask(taskname):
    # type: (String) -> UIResponse
    """Takes the name of a task as an argument as a string (must be
    configured on the Controller before hand), attempts to execute the
    task.

    This function should only be called from the Controller.

    To run in the client, the user needs a role-based permission. This
    permission is disabled by default.

    Note:
        If called from an Agent on 8.1.11+, this function will return an
        exception.

    Args:
        taskname: Name of the task to run. If more than one task has
            this name, an error will be returned.

    Returns:
        A UIResponse with a list of infos, errors, and warnings.
    """
    print(taskname)
    return UIResponse(Locale.ENGLISH)



__all__ = ["ScheduleModel"]

from com.palantir.ptoss.cinch.core import DefaultBindableModel


class ScheduleModel(DefaultBindableModel):
    def deselectDayOfWeek(self, day):
        pass

    def getEveryHour(self):
        pass

    def getHour(self):
        pass

    def getMinute(self):
        pass

    def getSelectedDays(self):
        pass

    def isAlldays(self):
        pass

    def isDaySelected(self, day):
        pass

    def isEveryHourEnabled(self):
        pass

    def isEveryMinuteEnabled(self):
        pass

    def isHourEnabled(self):
        pass

    def isMinuteEnabled(self):
        pass

    def selectDayOfWeek(self, day):
        pass

    def setAllDays(self, allDays):
        pass

    def setEveryHour(self, everyHour):
        pass

    def setHour(self, hour):
        pass

    def setHourEnabled(self, hourEnabled):
        pass

    def setMinute(self, minute):
        pass

    def setMinuteEnabled(self, minuteEnabled):
        pass

    def setSelectedDays(self, selectedDays):
        pass

    def toCronString(self):
        pass



"""Report Functions.

The following functions give you access to report details and the
ability to run reports.
"""

from __future__ import print_function

__all__ = [
    "executeAndDistribute",
    "executeReport",
    "getReportNamesAsDataset",
    "getReportNamesAsList",
]

from typing import Any, Dict, List, Optional

from com.inductiveautomation.ignition.common import BasicDataset
from java.lang import IllegalArgumentException, String


def executeAndDistribute(
    path,  # type: String
    project="project",  # type: String
    parameters=None,  # type: Optional[Dict[String, int]]
    action=None,  # type: Optional[String]
    actionSettings=None,  # type: Optional[Dict[String, Any]]
):
    # type: (...) -> None
    """Executes and distributes a report.

    Similar to scheduling a report to execute, except a schedule in not
    required to utilize this function. This is a great way to distribute
    the report on demand from a Client.

    Note:
        The function system.report.executeAndDistribute() does not run
        on its own thread and can get blocked. For example, if a printer
        is backed up and it takes a while to finish the request made by
        this function, the script will block the execution of other
        things on that thread until it finishes. Be sure to keep this in
        mind when using it in a script.

    Args:
        path: The path to the existing report.
        project: The name of the project where the report is located.
            Optional in client scope.
        parameters: An optional dictionary of parameter overrides, in
            the form name:value.
        action: The name of the distribution action to use.
        actionSettings: An optional dictionary of settings particular to
            the action. Missing values will use the default value for
            that action.

    Raises:
        IllegalArgumentException: Thrown when any of the following
            occurs: If the file type is not recognized, path does not
            exist, project does not exist, or a key is not valid.
    """
    if project is None:
        raise IllegalArgumentException()

    print(path, project, parameters, action, actionSettings)


def executeReport(
    path,  # type: String
    project="project",  # type: String
    parameters=None,  # type: Optional[Dict[String, int]]
    fileType="pdf",  # type: String
):
    # type: (...) -> Any
    """Immediately executes an existing report and returns a byte[] of
    the output.

    Args:
        path: The path to the existing report.
        project: The name of the project where the report is located.
            Optional in client scope.
        parameters: An optional dictionary of parameter overrides, in
            the form name:value. Optional.
        fileType: The file type the resulting byte array should
            represent. Defaults to "pdf". Not case-sensitive. Optional.

    Returns:
        A byte array of the resulting report.

    Raises:
        IllegalArgumentException: Thrown when any of the following
            occurs: If the file type is not recognized, path does not
            exist, project does not exist.
    """
    file_types = [
        "csv",
        "html",
        "jpeg",
        "pdf",
        "png",
        "rtf",
        "xls",
        "xlsx",
        "xml",
    ]
    if path is None or project is None or fileType not in file_types:
        raise IllegalArgumentException()
    print(path, project, parameters, fileType)


def getReportNamesAsDataset(project="project", includeReportName=True):
    # type: (Optional[String], Optional[bool]) -> BasicDataset
    """Gets a data of all reports for a project.

    Args:
        project: The name of the project where the reports are located.
            Optional in client scope.
        includeReportName: When set to False, the end of Path does not
            include the report name. Default is True. Optional.

    Returns:
        A dataset of report paths and names for the project. Returns an
        empty dataset if the project has no reports.

    Raises:
        IllegalArgumentException: Thrown when any of the following
            occurs: If the project name is omitted in the Gateway scope,
            project does not exist.
    """
    if project is None:
        raise IllegalArgumentException()

    print(project, includeReportName)
    return BasicDataset()


def getReportNamesAsList(project="project"):
    # type: (Optional[String]) -> List[String]
    """Gets a list of all reports for a project.

    Args:
        project: The name of the project where the reports are located.
            Optional in client scope.

    Returns:
        A list of report paths for the project. Returns an empty list if
        the project has no reports.

    Raises:
        IllegalArgumentException: Thrown when any of the following
            occurs: If the project name is omitted in the Gateway scope,
            project does not exist.
    """
    if project is None:
        raise IllegalArgumentException()
    return []



"""The Ignition scripting API, which is available under the module name
"system", is full of functions that are useful when designing projects
in Ignition. From running database queries, manipulating components, to
exporting data, scripting functions can help. Some of these functions
only work in the Gateway scope, and other only work in the Client scope,
while the rest will work in any scope.

Additional information on scripting Ignition can be found in the
Scripting section.
"""



from __future__ import print_function

__all__ = [
    "Icon",
    "JComponent",
    "JDesktopPane",
    "JFrame",
    "JInternalFrame",
    "JLabel",
    "JLayeredPane",
    "JOptionPane",
    "JPanel",
    "JPopupMenu",
    "JTextField",
]

from typing import Any, List, Optional

from java.awt import Component, Container, Frame, Graphics
from java.lang import String
from java.util import Locale
from javax.swing.event import AncestorListener
from javax.swing.plaf import DesktopPaneUI
from javax.swing.text import JTextComponent


class DesktopManager(object):
    def activateFrame(self, f):
        # type: (JInternalFrame) -> None
        raise NotImplementedError

    def beginDragginFrame(self, f):
        # type: (JComponent) -> None
        raise NotImplementedError

    def beginResizingFrame(self, f, direction):
        # type: (JComponent, int) -> None
        raise NotImplementedError

    def closeFrame(self, f):
        # type: (JInternalFrame) -> None
        raise NotImplementedError

    def deactivateFrame(self, f):
        # type: (JInternalFrame) -> None
        raise NotImplementedError

    def deiconifyFrame(self, f):
        # type: (JInternalFrame) -> None
        raise NotImplementedError

    def dragFrame(self, f, newX, newY):
        # type: (JComponent, int, int) -> None
        raise NotImplementedError

    def endDraggingFrame(self, f):
        # type: (JComponent) -> None
        raise NotImplementedError

    def endResizing(self, f):
        # type: (JComponent) -> None
        raise NotImplementedError

    def iconifyFrame(self, f):
        # type: (JInternalFrame) -> None
        raise NotImplementedError

    def maximizeFrame(self, f):
        # type: (JInternalFrame) -> None
        raise NotImplementedError

    def minimizeFrame(self, f):
        # type: (JInternalFrame) -> None
        raise NotImplementedError

    def openFrame(self, f):
        # type: (JInternalFrame) -> None
        raise NotImplementedError

    def resizeFrame(self, f, newX, newY, newWidth, newHeight):
        # type: (JComponent, int, int, int, int) -> None
        raise NotImplementedError

    def setBoundsForFrame(self, f, newX, newY, newWidth, newHeight):
        # type: (JComponent, int, int, int, int) -> None
        raise NotImplementedError


class Icon(object):
    def getIconHeight(self):
        # type: () -> int
        raise NotImplementedError

    def getIconWidth(self):
        # type: () -> int
        raise NotImplementedError

    def paintIcon(self, c, g, x, y):
        # type: (Component, Graphics, int, int) -> None
        raise NotImplementedError


class JComponent(Container):
    TOOL_TIP_TEXT_KEY = None  # type: String
    UNDEFINED_CONDITION = None  # type: int
    WHEN_ANCESTOR_OF_FOCUSED_COMPONENT = None  # type: int
    WHEN_FOCUSED = None  # type: int
    WHEN_IN_FOCUSED_WINDOW = None  # type: int

    def __init__(self):
        # type: () -> None
        pass

    def addAncestorListener(self, listener):
        # type: (AncestorListener) -> None
        pass

    def addNotify(self):
        # type: () -> None
        pass

    def createToolTip(self):
        # type: () -> JToolTip
        pass

    @staticmethod
    def getDefaultLocale():
        # type: () -> Locale
        pass

    @staticmethod
    def isLightweightComponent(c):
        # type: (Component) -> bool
        pass

    @staticmethod
    def setDefaultLocale(l):
        # type: (Locale) -> None
        pass


class JFrame(Frame):
    def __init__(self, *args):
        # type: (*Any) -> None
        super(JFrame, self).__init__(*args)


class JInternalFrame(JComponent):
    def __init__(
        self,
        title=None,  # type: Optional[String]
        resizable=None,  # type: Optional[bool]
        closable=None,  # type: Optional[bool]
        maximizable=None,  # type: Optional[bool]
        iconifiable=None,  # type: Optional[bool]
    ):
        # type: (...) -> None
        super(JInternalFrame, self).__init__()
        print(title, resizable, closable, maximizable, iconifiable)


class JLabel(JComponent):
    """A display area for a short text string or an image, or both."""

    def __init__(self, *args):
        # type: (*Any) -> None
        super(JLabel, self).__init__()
        print(args)


class JLayeredPane(JComponent):
    DEFAULT_LAYER = 0
    DRAG_LAYER = 400
    FRAME_CONTENT_LAYER = -30000
    LAYER_PROPERTY = "layeredContainerLayer"
    MODAL_LAYER = 200
    PALETTE_LAYER = 100
    POPUP_LAYER = 300

    def __init__(self):
        # type: () -> None
        super(JLayeredPane, self).__init__()

    def getComponentCountInLayer(self, layer):
        # type: (int) -> int
        pass

    def getLayer(self, c):
        # type: (Component) -> int
        pass

    def highestLayer(self):
        # type: () -> int
        pass

    def lowestLayer(self):
        # type: () -> int
        pass

    def setPosition(self, c, position):
        # type: (Component, int) -> None
        pass


class JDesktopPane(JLayeredPane):
    LIVE_DRAG_MODE = None  # type: int
    OUTLINE_DRAG_MODE = None  # type: int

    def __init__(self):
        # type: () -> None
        super(JDesktopPane, self).__init__()

    def getAllFrames(self):
        # type: () -> List[JInternalFrame]
        pass

    def getAllFramesInLayer(self, layer):
        # type: (int) -> List[JInternalFrame]
        pass

    def getDesktopManager(self):
        # type: () -> DesktopManager
        pass

    def getDragMode(self):
        # type: () -> int
        pass

    def getSelectedFrame(self):
        # type: () -> JInternalFrame
        pass

    def getUI(self):
        # type: () -> DesktopPaneUI
        pass

    def getUIClassID(self):
        # type: () -> String
        pass

    def updateUI(self):
        # type: () -> None
        pass


class JOptionPane(JComponent):
    """JOptionPane makes it easy to pop up a standard dialog box that
    prompts users for a value or informs them of something. For
    information about using JOptionPane, see How to Make Dialogs, a
    section in The Java Tutorial.
    """

    # messageType.
    PLAIN_MESSAGE = -1
    ERROR_MESSAGE = 0
    INFORMATION_MESSAGE = 1
    WARNING_MESSAGE = 2
    QUESTION_MESSAGE = 3

    # optionType.
    DEFAULT_OPTION = -1
    YES_NO_OPTION = 0
    YES_NO_CANCEL_OPTION = 1
    OK_CANCEL_OPTION = 2

    # When one of the showXxxDialog methods returns an integer, the
    # possible values are:
    CLOSED_OPTION = -1
    OK_OPTION = 0
    YES_OPTION = 0
    NO_OPTION = 1
    CANCEL_OPTION = 2

    @staticmethod
    def showConfirmDialog(
        parentComponent,  # type: Optional[Any]
        message,  # type: Any
        title=None,  # type: Optional[String]
        optionType=None,  # type: Optional[int]
        messageType=None,  # type: Optional[int]
        icon=None,  # type: Optional[Icon]
    ):
        # type: (...) -> int
        print(parentComponent, message, title, optionType, messageType, icon)
        return JOptionPane.YES_OPTION

    @staticmethod
    def showInputDialog(
        parentComponent,  # type: Optional[Any]
        message,  # type: Any
        title=None,  # type: Optional[String]
        messageType=None,  # type: Optional[int]
        icon=None,  # type: Optional[Icon]
        selectionValues=None,  # type: Optional[List[Any]]
        initialSelectionValue=None,  # type: Optional[Any]
    ):
        # type: (...) -> String
        print(
            parentComponent,
            message,
            title,
            messageType,
            icon,
            selectionValues,
            initialSelectionValue,
        )
        return "Input"

    @staticmethod
    def showMessageDialog(
        parentComponent,  # type: Optional[Any]
        message,  # type: Any
        title=None,  # type: Optional[String]
        messageType=None,  # type: Optional[int]
        icon=None,  # type: Optional[Icon]
    ):
        # type: (...) -> None
        print(parentComponent, message, title, messageType, icon)

    @staticmethod
    def showOptionDialog(
        parentComponent,  # type: Optional[Any]
        message,  # type: Any
        title=None,  # type: Optional[String]
        optionType=None,  # type: Optional[int]
        messageType=None,  # type: Optional[int]
        icon=None,  # type: Optional[Icon]
        options=None,  # type: Optional[List[Any]]
        initialValue=None,  # type: Optional[Any]
    ):
        # type: (...) -> int
        print(
            parentComponent,
            message,
            title,
            optionType,
            messageType,
            icon,
            options,
            initialValue,
        )
        return JOptionPane.YES_OPTION


class JPanel(JComponent):
    def __init__(self, *args):
        # type: (*Any) -> None
        super(JPanel, self).__init__()
        print(args)


class JToolTip(JComponent):
    def __init__(self):
        # type: () -> None
        super(JToolTip, self).__init__()

    def getComponent(self):
        # type: () -> JComponent
        pass

    def getTipText(self):
        # type: () -> String
        pass


class JPopupMenu(JComponent):
    """An implementation of a popup menu -- a small window that pops up
    and displays a series of choices. A JPopupMenu is used for the menu
    that appears when the user selects an item on the menu bar. It is
    also used for "pull-right" menu that appears when the selects a menu
    item that activates it. Finally, a JPopupMenu can also be used
    anywhere else you want a menu to appear. For example, when the user
    right-clicks in a specified area.
    """

    def __init__(self, label=None):
        # type: (Optional[String]) -> None
        super(JPopupMenu, self).__init__()
        print(label)


class JTextField(JTextComponent):
    """JTextField is a lightweight component that allows the editing of
    a single line of text.
    """

    def __init__(self, *args):
        # type: (*Any) -> None
        pass



from __future__ import print_function

__all__ = ["Result", "Results"]

from typing import Any, List, Optional, Sequence

from com.inductiveautomation.ignition.common.model.values import QualityCode
from java.lang import Object, String


class Result(object):
    def getDisplayPath(self):
        raise NotImplementedError

    def getPath(self):
        raise NotImplementedError

    def getType(self):
        raise NotImplementedError

    def hasChildren(self):
        raise NotImplementedError


class Results(Object):
    """The results of a browse operation.

    May only represent a partial result set, which can be determined by
    comparing the Total Available Size to the Returned Size. If there is
    a mismatch, the continuation point should be non-null and can be
    used in constructing the subsequent BrowseFilter to continue the
    browse.
    """

    continuationPoint = None  # type: Optional[String]
    resultQuality = None  # type: QualityCode
    results = None  # type: Sequence[Any]
    totalAvailableResults = None  # type: int

    def __init__(self, *args):
        # type: (*Any) -> None
        print(args)
        self.continuationPoint = None
        self.resultQuality = QualityCode.Good
        self.results = []
        self.totalAvailableResults = 0

    @staticmethod
    def error(result):
        # type: (Results) -> Results
        return Results(result)

    def getContinuationPoint(self):
        # type: () -> Optional[String]
        return self.continuationPoint

    def getResultQuality(self):
        # type: () -> QualityCode
        return self.resultQuality

    def getResults(self):
        # type: () -> Sequence[Any]
        return self.results

    def getReturnedSize(self):
        # type: () -> int
        return 0 if self.results is None else len(self.results)

    def getTotalAvailableSize(self):
        # type: () -> int
        return self.totalAvailableResults

    @staticmethod
    def of(results):
        # type: (List[Any]) -> Results
        return Results(results)

    def setContinuationPoint(self, continuationPoint=None):
        # type: (Optional[String]) -> None
        self.continuationPoint = continuationPoint

    def setResultQuality(self, value):
        # type: (QualityCode) -> None
        self.resultQuality = value

    def setResults(self, results):
        # type: (List[Any]) -> None
        self.results = results

    def setTotalAvailableResults(self, totalAvailableResults):
        # type: (int) -> None
        self.totalAvailableResults = totalAvailableResults



__all__ = [
    "CopyOption",
    "DirectoryStream",
    "FileStore",
    "FileVisitOption",
    "Files",
    "LinkOption",
    "OpenOption",
    "Path",
    "Paths",
    "StandardCopyOption",
    "WatchEvent",
    "WatchKey",
    "Watchable",
]

from typing import Any, Iterator, List, Optional, Set, TypeVar, Union

from java.io import BufferedReader, BufferedWriter, InputStream, OutputStream
from java.lang import AutoCloseable, CharSequence, Class, Enum, Object, String
from java.nio.channels import SeekableByteChannel
from java.nio.charset import Charset
from java.nio.file.attribute import (
    BasicFileAttributes,
    FileAttribute,
    FileAttributeView,
    FileStoreAttributeView,
    FileTime,
    PosixFilePermission,
    UserPrincipal,
)
from java.util.function import BiPredicate
from java.util.stream import Stream

T = TypeVar("T")


class CopyOption(object):
    pass


class DirectoryStream(AutoCloseable):
    def close(self):
        # type: () -> None
        pass


class OpenOption(object):
    pass


class Watchable(object):
    def register(self, *args):
        # type: (Any) -> WatchKey
        raise NotImplementedError


class FileStore(Object):
    def getAttribute(self, attribute):
        raise NotImplementedError

    def getBlockSize(self):
        # type: () -> long
        pass

    def getFileStoreAttributeView(self, type):
        # type: (Class) -> FileStoreAttributeView
        raise NotImplementedError

    def getTotalSpace(self):
        # type: () -> long
        raise NotImplementedError

    def getUnallocatedSpace(self):
        # type: () -> long
        raise NotImplementedError

    def getUsableSpace(self):
        # type: () -> long
        raise NotImplementedError

    def isReadOnly(self):
        # type: () -> bool
        raise NotImplementedError

    def name(self):
        # type: () -> String
        raise NotImplementedError

    def supportsFileAttributeView(self, type):
        # type: (Class) -> bool
        raise NotImplementedError

    def type(self):
        # type: () -> String
        raise NotImplementedError


class FileVisitOption(Enum):
    FOLLOW_LINKS = None  # type: FileVisitOption

    @staticmethod
    def values():
        # type: () -> List[FileVisitOption]
        pass


class Files(Object):
    @staticmethod
    def copy(*args):
        # type: (*Any) -> Union[long, Path]
        pass

    @staticmethod
    def createDirectories(dir, *attrs):
        # type: (Path, *FileAttribute) -> Path
        pass

    @staticmethod
    def createDirectory(dir, *attrs):
        # type: (Path, *FileAttribute) -> Path
        pass

    @staticmethod
    def createFile(path, *attrs):
        # type: (Path, *FileAttribute) -> Path
        pass

    @staticmethod
    def createLink(link, existing):
        # type: (Path, Path) -> Path
        pass

    @staticmethod
    def createSymbolicLink(link, target, *attrs):
        # type: (Path, Path, FileAttribute) -> Path
        pass

    @staticmethod
    def createTempDirectory(*args):
        # type: (*Any) -> Path
        pass

    @staticmethod
    def createTempFile(*args):
        # type: (*Any) -> Path
        pass

    @staticmethod
    def delete(path):
        # type: (Path) -> None
        pass

    @staticmethod
    def deleteIfExists(path):
        # type: (Path) -> bool
        pass

    @staticmethod
    def exists(path, *options):
        # type: (Path, *LinkOption) -> bool
        pass

    @staticmethod
    def find(start, maxDepth, matcher, *options):
        # type: (Path, int, BiPredicate, *FileVisitOption) -> Stream
        pass

    @staticmethod
    def getAttribute(path, attribute, *options):
        # type: (Path, String, *LinkOption) -> Object
        pass

    @staticmethod
    def getFileAttributeView(path, type, *options):
        # type: (Path, Class, *LinkOption) -> FileAttributeView
        pass

    @staticmethod
    def getFileStore(path):
        # type: (Path) -> FileStore
        pass

    @staticmethod
    def getLastModifiedTime(path, *options):
        # type: (Path, *LinkOption) -> FileTime
        pass

    @staticmethod
    def getOwner(path, *options):
        # type: (Path, *LinkOption) -> UserPrincipal
        pass

    @staticmethod
    def getPosixFilePermissions(path, *options):
        # type: (Path, *LinkOption) -> Set[PosixFilePermission]
        pass

    @staticmethod
    def isDirectory(path, *options):
        # type: (Path, *LinkOption) -> bool
        pass

    @staticmethod
    def isExecutable(path):
        # type: (Path) -> bool
        pass

    @staticmethod
    def isHidden(path):
        # type: (Path) -> bool
        pass

    @staticmethod
    def isReadable(path):
        # type: (Path) -> bool
        pass

    @staticmethod
    def isRegularFile(path, *options):
        # type: (Path, *LinkOption) -> bool
        pass

    @staticmethod
    def isSameFile(path, path2):
        # type: (Path, Path) -> bool
        pass

    @staticmethod
    def isSymbolicLink(path):
        # type: (Path) -> bool
        pass

    @staticmethod
    def isWritable(path):
        # type: (Path) -> bool
        pass

    @staticmethod
    def lines(path, cs=None):
        # type: (Path, Optional[Charset]) -> Stream
        pass

    @staticmethod
    def list(path):
        # type: (Path) -> Stream
        pass

    @staticmethod
    def move(source, target, *options):
        # type: (Path, Path, *CopyOption) -> Path
        pass

    @staticmethod
    def newBufferedReader(path, cs=None):
        # type: (Path, Optional[Charset]) -> BufferedReader
        pass

    @staticmethod
    def newBufferedWriter(path, *args):
        # type: (Path, *Any) -> BufferedWriter
        pass

    @staticmethod
    def newByteChannel(path, *args):
        # type: (Path, *Any) -> SeekableByteChannel
        pass

    @staticmethod
    def newDirectoryStream(dir, *args):
        # type: (Path, *Any) -> DirectoryStream
        pass

    @staticmethod
    def newInputStream(path, *options):
        # type: (Path, *OpenOption) -> InputStream
        pass

    @staticmethod
    def newOutputStream(path, *options):
        # type: (Path, *OpenOption) -> OutputStream
        pass

    @staticmethod
    def notExists(path, *options):
        # type: (Path, *LinkOption) -> bool
        pass

    @staticmethod
    def probeContentType(path):
        # type: (Path) -> String
        pass

    @staticmethod
    def readAllBytes(path):
        # type: (Path) -> bytearray
        pass

    @staticmethod
    def readAllLines(path, cs=None):
        # type: (Path, Optional[Charset]) -> List[String]
        pass

    @staticmethod
    def readAttributes(path, *args):
        # type: (Path, *Any) -> BasicFileAttributes
        pass

    @staticmethod
    def readString(path, cs=None):
        # type: (Path, Optional[Charset]) -> String
        pass

    @staticmethod
    def readSymbolicLink(link):
        # type: (Path) -> Path
        pass

    @staticmethod
    def setAttribute(path, attribute, value, *options):
        # type: (Path, String, Object, *LinkOption) -> Path
        pass

    @staticmethod
    def setLastModifiedTime(path, time):
        # type: (Path, FileTime) -> Path
        pass

    @staticmethod
    def setOwner(path, owner):
        # type: (Path, UserPrincipal) -> Path
        pass

    @staticmethod
    def setPosixFilePermissions(path, perms):
        # type: (Path, Set[PosixFilePermission]) -> Path
        pass

    @staticmethod
    def size(path):
        # type: (Path) -> long
        pass

    @staticmethod
    def walk(path, *args):
        # type: (Path, *Any) -> Stream
        pass

    @staticmethod
    def walkFileTree(path, *args):
        # type: (Path, *Any) -> Path
        pass

    @staticmethod
    def write(path, *args):
        # type: (Path, *Any) -> Path
        pass

    @staticmethod
    def writeString(path, csq, *args):
        # type: (Path, CharSequence, *Any) -> Path
        pass


class LinkOption(Enum, CopyOption, OpenOption):
    NOFOLLOW_LINKS = None  # type: LinkOption

    @staticmethod
    def values():
        # type: () -> List[LinkOption]
        pass


class Path(Watchable):
    def compareTo(self, other):
        # type: (Path) -> int
        pass

    def endsWith(self, other):
        # type: (Union[Object, Path, String]) -> bool
        pass

    def equals(self, other):
        # type: (Object) -> bool
        pass

    def getFileName(self):
        # type: () -> Path
        pass

    def getName(self, index):
        # type: (int) -> Path
        pass

    def getNameCount(self):
        # type: () -> int
        pass

    def getParent(self):
        # type: () -> Path
        pass

    def getRoot(self):
        # type: () -> Path
        pass

    def hashCode(self):
        # type: () -> int
        pass

    def isAbsolute(self):
        # type: () -> bool
        pass

    def iterator(self):
        # type: () -> Iterator[Path]
        pass

    def normalize(self):
        # type: () -> Path
        pass

    @staticmethod
    def of(*args):
        # type: (Any) -> Path
        pass

    def register(self, *args):
        # type: (Any) -> WatchKey
        pass

    def relativize(self, other):
        # type: (Path) -> Path
        pass

    def resolve(self, other):
        # type: (Union[Path, String]) -> Path
        pass

    def resolveSibling(self, other):
        # type: (Path) -> Path
        pass

    def startsWith(self, other):
        # type: (Path) -> bool
        pass

    def subpath(self, beginIndex, endIndex):
        # type: (int, int) -> Path
        pass

    def toAbsolutePath(self):
        # type: () -> Path
        pass

    def toFile(self):
        # type: () -> Any
        pass

    def toRealPath(self, *args):
        # type: (LinkOption) -> Path
        pass

    def toString(self):
        # type: () -> String
        pass

    def toUri(self):
        # type: () -> Any
        pass


class Paths(Object):
    @staticmethod
    def get(*args):
        # type: (*Any) -> Path
        pass


class StandardCopyOption(Enum, CopyOption, OpenOption):
    ATOMIC_MOVE = None  # type: StandardCopyOption
    COPY_ATTRIBUTES = None  # type: StandardCopyOption
    REPLACE_EXISTING = None  # type: StandardCopyOption

    @staticmethod
    def values():
        # type: () -> List[StandardCopyOption]
        pass


class WatchEvent(object):
    def context(self):
        # type: () -> T
        raise NotImplementedError

    def count(self):
        # type: () -> int
        raise NotImplementedError

    def kind(self):
        # type: () -> WatchEvent.Kind
        raise NotImplementedError

    class Kind(object):
        def name(self):
            # type: () -> String
            raise NotImplementedError

        def type(self):
            # type: () -> Class
            raise NotImplementedError

    class Modifier(object):
        def name(self):
            # type: () -> String
            raise NotImplementedError


class WatchKey(object):
    def cancel(self):
        # type: () -> None
        raise NotImplementedError

    def isValid(self):
        # type: () -> bool
        raise NotImplementedError

    def pollEvents(self):
        # type: () -> List[WatchEvent]
        raise NotImplementedError

    def reset(self):
        # type: () -> bool
        raise NotImplementedError

    def watchable(self):
        # type: () -> Watchable
        raise NotImplementedError



from typing import List

from java.lang import Enum


class RoundingMode(Enum):
    CEILING = None  # type: RoundingMode
    DOWN = None  # type: RoundingMode
    FLOOR = None  # type: RoundingMode
    HALF_DOWN = None  # type: RoundingMode
    HALF_EVEN = None  # type: RoundingMode
    HALF_UP = None  # type: RoundingMode
    UNNECESSARY = None  # type: RoundingMode
    UP = None  # type: RoundingMode

    @staticmethod
    def values():
        # type: () -> List[RoundingMode]
        pass



"""OPC Functions.

The following functions allow you to read, write and browser OPC
servers.
"""

from __future__ import print_function

__all__ = [
    "browse",
    "browseServer",
    "browseSimple",
    "getServerState",
    "getServers",
    "isServerEnabled",
    "readValue",
    "readValues",
    "setServerEnabled",
    "writeValue",
    "writeValues",
]

from typing import Any, List, Optional, Union

from com.inductiveautomation.ignition.common.model.values import (
    BasicQualifiedValue,
    QualityCode,
)
from com.inductiveautomation.ignition.common.opc import BasicOPCBrowseElement
from com.inductiveautomation.ignition.common.script.builtin import AbstractOPCUtilities
from com.inductiveautomation.ignition.common.script.builtin.ialabs import OPCBrowseTag
from java.lang import String

BrowseServerResult = List[Union[BasicOPCBrowseElement, AbstractOPCUtilities.PyOPCTag]]


def browse(opcServer, device, folderPath, opcItemPath):
    # type: (String, String, String, String) -> List[OPCBrowseTag]
    """Allows browsing of the OPC Servers in the runtime, returning a
    list of Tags.

    Args:
        opcServer: The name of the OPC Server to browse.
        device: The name of the device to browse.
        folderPath: Filters on a folder path. Use * as a wildcard for
            any number of characters and a ? for a single character.
        opcItemPath: Filters on a OPC item path. Use * as a wildcard for
            any number of characters and a ? for a single character.

    Returns:
        An array of OPCBrowseTag objects.
    """
    print(opcServer, device, folderPath, opcItemPath)
    return [OPCBrowseTag()]


def browseServer(
    opcServer,  # type: String
    nodeId,  # type: String
):
    # type: (...) -> BrowseServerResult
    """When called from a Vision Client, returns a list of
    OPCBrowseElement objects for the given Server. Otherwise, returns a
    list of PyOPCTag.

    Args:
        opcServer: The name of the OPC Server connection.
        nodeId: The node ID to browse.

    Returns:
        A list of OPCBrowseElement/PyOPCTag objects.
    """
    print(opcServer, nodeId)
    return [BasicOPCBrowseElement()]


def browseSimple(opcServer, device, folderPath, opcItemPath):
    # type: (String, String, String, String) -> List[OPCBrowseTag]
    """Allows browsing of OPC servers in the runtime returning a list of
    tags.

    browseSimple() takes mandatory parameters, which can be null, while
    browse() uses keyword-style arguments.

    Note:
        The spelling on the opcServer and device parameters must be
        exact.

    Args:
        opcServer: The name of the OPC Server to browse.
        device: The name of the device to browse.
        folderPath: Filters on a folder path. Use * as a wildcard for
            any number of characters and a ? for a single character.
        opcItemPath: Filters on a OPC item path. Use * as a wildcard for
            any number of characters and a ? for a single character.

    Returns:
        An array of OPCBrowseTag objects.
    """
    print(opcServer, device, folderPath, opcItemPath)
    return [OPCBrowseTag()]


def getServerState(opcServer):
    # type: (String) -> Optional[String]
    """Retrieves the current state of the given OPC Server connection.

    If the given Server is not found, the return value will be None.
    Otherwise, the return value will be one of these strings:
        - UNKNOWN
        - FAULTED
        - CONNECTING
        - CLOSED
        - CONNECTED
        - DISABLED

    Args:
        opcServer: The name of an OPC Server connection.

    Returns:
        A string representing the current state of the connection, or
        None if the connection doesn't exist.
    """
    print(opcServer)
    return "CONNECTED"


def getServers(includeDisabled=False):
    # type: (Optional[bool]) -> List[String]
    """Returns a list of Server names.

    Args:
        includeDisabled: If set to True, enabled and disabled servers
            will be returned. If set to False, only enabled servers will
            be returned. Defaults to False. Optional.

    Returns:
        A list of Server name strings. If no servers are found, returns
        an empty list.
    """
    print(includeDisabled)
    return ["serverName"]


def isServerEnabled(serverName):
    # type: (String) -> bool
    """Checks if an OPC Server connection is enabled or disabled.

    Args:
        serverName: The name of an OPC Server connection.

    Returns:
        True if the connection is enabled, False if the connection is
        disabled.
    """
    print(serverName)
    return True


def readValue(opcServer, itemPath):
    # type: (String, String) -> BasicQualifiedValue
    """Reads a single value directly from an OPC Server connection.

    The address is specified as a string, for example,
    [MyDevice]N11/N11:0. The object returned from this function has
    three attributes: value, quality, and timestamp. The value attribute
    represents the current value for the address specified.

    The quality attribute is an OPC-UA status code. You can easily check
    a good quality vs a bad quality by calling the isGood() function on
    the quality object. The timestamp attribute is Date object that
    represents the time that the value was retrieved at.

    Args:
        opcServer: The name of the OPC Server connection in which the
            item resides.
        itemPath: The item path, or address, to read from.

    Returns:
        An object that contains the value, quality, and timestamp
        returned from the OPC Server for the address specified.
    """
    print(opcServer, itemPath)
    return BasicQualifiedValue()


def readValues(opcServer, itemPaths):
    # type: (String, List[String]) -> List[BasicQualifiedValue]
    """This function is equivalent to the system.opc.readValue function,
    except that it can operate in bulk.

    You can specify a list of multiple addresses to read from, and you
    will receive a list of the same length, where each entry is the
    qualified value object for the corresponding address.

    Args:
        opcServer: The name of the OPC Server connection in which the
            items reside.
        itemPaths: A list of strings, each representing an item path, or
            address to read from.

    Returns:
        A sequence of objects, one for each address specified, in order.
        Each object will contain the value, quality, and timestamp
        returned from the OPC Server for the corresponding address.
    """
    print(opcServer, itemPaths)
    return [BasicQualifiedValue() for _ in itemPaths]


def setServerEnabled(serverName, enabled):
    # type: (String, bool) -> None
    """Enables or disables an OPC Server connection.

    Args:
        serverName: The name of an OPC Server connection.
        enabled: The new state the connection should be set to: True to
            enable the connection, False to disable.
    """
    print(serverName, enabled)


def writeValue(opcServer, itemPath, value):
    # type: (String, String, Any) -> QualityCode
    """Writes a value directly through an OPC Server connection
    synchronously.

    Will return an OPC-UA status code object. You can quickly check if
    the write succeeded by calling isGood() on the return value from
    this function.

    Args:
        opcServer: The name of the OPC Server connection in which the
            item resides.
        itemPath: The item path, or address, to write to.
        value: The value to write to the OPC item.

    Returns:
        The status of the write. Use returnValue.isGood() to check if
        the write succeeded.
    """
    print(opcServer, itemPath, value)
    return QualityCode()


def writeValues(opcServer, itemPaths, values):
    # type: (String, List[String], List[Any]) -> List[QualityCode]
    """This function is a bulk version of system.opc.writeValue.

    It takes a list of addresses and a list of objects, which must be
    the same length. It will write the corresponding object to the
    corresponding address in bulk. It will return a list of status codes
    representing the individual write success or failure for each
    corresponding address.

    Args:
        opcServer: The name of the OPC Server connection in which the
            items reside.
        itemPaths: A list of item paths, or addresses, to write to.
        values: A list of values to write to each address specified.

    Returns:
        An array of Quality objects, each entry corresponding in order
        to the addresses specified.
    """
    print(opcServer, itemPaths, values)
    return [QualityCode() for _ in itemPaths]



from __future__ import print_function

__all__ = [
    "AbstractTagPath",
    "BasicTagPath",
    "PropertyAlteredTagPath",
    "SourceAlteredTagPath",
]

from typing import Any, List, Optional, Union

from com.inductiveautomation.ignition.common.config import Property
from com.inductiveautomation.ignition.common.tags.model import TagPath
from java.lang import Object, String


class AbstractTagPath(Object, TagPath):
    @staticmethod
    def compareNullLow(c1, c2):
        # type: (Property, Property) -> int
        pass


class BasicTagPath(AbstractTagPath):
    def __init__(
        self,
        source,  # type: String
        pathParts=None,  # type: Optional[List[String]]
        prop=None,  # type: Optional[Property]
    ):
        # type: (...) -> None
        print(source, pathParts, prop)

    @staticmethod
    def append(root, arg):
        # type: (TagPath, Union[String, TagPath]) -> TagPath
        pass

    @staticmethod
    def copy(path):
        # type: (TagPath) -> BasicTagPath
        pass

    @staticmethod
    def renameParentFolder(path, newParent):
        # type: (TagPath, TagPath) -> BasicTagPath
        pass

    @staticmethod
    def subPath(path, *args):
        # type: (TagPath, *Any) -> BasicTagPath
        pass


class PropertyAlteredTagPath(AbstractTagPath):
    def __init__(self, path, prop):
        # type: (TagPath, Property) -> None
        print(path, prop)


class SourceAlteredTagPath(AbstractTagPath):
    def __init__(self, path, source):
        # type: (TagPath, String) -> None
        print(path, source)



from __future__ import print_function

__all__ = ["EventProperty", "EventPropertyType", "ShelvedPath"]

from typing import Any, List

from com.inductiveautomation.ignition.common import Path, QualifiedPath
from com.inductiveautomation.ignition.common.config import Property
from java.lang import Class, Enum, Object, String
from java.util import Date


class EventProperty(Object):
    @staticmethod
    def create(prop, type):
        # type: (Property, EventPropertyType) -> EventProperty
        pass

    @staticmethod
    def createDynamic(prop, isConfig):
        # type: (Property, bool) -> EventProperty
        pass

    @staticmethod
    def createStatic(prop, isConfig):
        # type: (Property, bool) -> EventProperty
        pass

    def getDefaultValue(self):
        # type: () -> Object
        pass

    def getName(self):
        # type: () -> String
        pass

    def getPropertyType(self):
        # type: () -> EventPropertyType
        pass

    def getType(self):
        # type: () -> Class
        pass


class EventPropertyType(Enum):
    @staticmethod
    def values():
        # type: () -> List[EventPropertyType]
        pass


class ShelvedPath(Object):
    """This class provides information about a path that has been
    "shelved", such as when it was shelved, and by whom, and the actual
    path.
    """

    def __init__(self, *args):
        # type: (*Any) -> None
        print(args)

    def getExpiration(self):
        # type: () -> Date
        pass

    def getHitCount(self):
        # type: () -> int
        pass

    def getPath(self):
        # type: () -> Path
        pass

    def getShelveTime(self):
        # type: () -> Date
        pass

    def getUser(self):
        # type: () -> QualifiedPath
        pass

    def incrementHitCount(self):
        # type: () -> None
        pass

    def isExpired(self):
        # type: () -> bool
        pass



from typing import List

from com.inductiveautomation.ignition.common.user import User
from com.palantir.ptoss.cinch.core import DefaultBindableModel
from java.lang import String


class RosterModel(DefaultBindableModel):
    def getName(self):
        # type: () -> String
        pass

    def getUsers(self):
        # type: () -> List[String]
        pass

    def set(self, that):
        # type: (RosterModel) -> None
        pass

    def setName(self, name):
        # type: (String) -> None
        pass

    def setUsers(self, users):
        # type: (List[User]) -> None
        pass



from __future__ import print_function

__all__ = [
    "ExclusionStrategy",
    "FieldAttributes",
    "FieldNamingPolicy",
    "FieldNamingStrategy",
    "Gson",
    "GsonBuilder",
    "JsonArray",
    "JsonElement",
    "JsonPrimitive",
    "LongSerializationPolicy",
    "TypeAdapter",
    "TypeAdapterFactory",
]

from typing import Any, Iterable, Optional, TypeVar, Union

from com.inductiveautomation.ignition.common.gson.reflect import TypeToken
from com.inductiveautomation.ignition.common.gson.stream import JsonReader, JsonWriter
from java.io import Reader, Writer
from java.lang import Class, Enum, Object, String

T = TypeVar("T")


class ExclusionStrategy(object):
    def shouldSkipClass(self, clazz):
        # type: (Class) -> bool
        raise NotImplementedError

    def shouldSkipField(self, attributes):
        # type: (FieldAttributes) -> bool
        raise NotImplementedError


class FieldNamingStrategy(object):
    def translateName(self, arg):
        # type: (Any) -> String
        raise NotImplementedError


class TypeAdapterFactory(object):
    def create(self, gson, token):
        # type: (Gson, TypeToken) -> TypeAdapter
        raise NotImplementedError


class FieldAttributes(Object):
    def __init__(self, *args):
        # type: (*Any) -> None
        pass

    def getAnnotation(self, annotation):
        # type: (Class) -> T
        pass

    def getAnnotations(self):
        # type: () -> Iterable[Any]
        pass

    def getDeclaredClass(self):
        # type: () -> Class
        pass

    def getDeclaredType(self):
        # type: () -> Any
        pass

    def getDeclaringClass(self):
        # type: () -> Class
        pass

    def getName(self):
        # type: () -> String
        pass

    def hasModifier(self, modifier):
        # type: (int) -> bool
        pass

    def isSynthetic(self):
        # type: () -> bool
        pass


class FieldNamingPolicy(Enum, FieldNamingStrategy):
    IDENTITY = None  # type: String
    UPPER_CAMEL_CASE = None  # type: String
    UPPER_CAMEL_CASE_WITH_SPACES = None  # type: String
    LOWER_CASE_WITH_UNDERSCORES = None  # type: String
    LOWER_CASE_WITH_DASHES = None  # type: String
    LOWER_CASE_WITH_DOTS = None  # type: String

    def translateName(self, arg):
        # type: (Any) -> String
        pass


class Gson(Object):
    def excluder(self):
        # type: () -> Any
        pass

    def fieldNamingStrategy(self):
        # type: () -> FieldNamingStrategy
        pass

    def fromJson(self, *args):
        # type: (*Any) -> T
        pass

    def getAdapter(self, TypeToken=None):
        # type: (TypeToken) -> TypeAdapter
        pass

    def getDelegateAdapter(self, skipPast, type):
        # type: (TypeAdapterFactory, TypeToken) -> TypeAdapter
        pass

    def htmlSafe(self):
        # type: () -> bool
        pass

    def newBuilder(self):
        # type: () -> GsonBuilder
        pass

    def newJsonReader(self, reader):
        # type: (Reader) -> JsonReader
        pass

    def newJsonWriter(self, writer):
        # type: (Writer) -> JsonWriter
        pass

    def serializeNulls(self):
        # type: () -> bool
        pass

    def toJson(self, *args):
        # type: (*Any) -> Optional[String]
        pass

    def toJsonTree(self, src, typeOfSrc=None):
        # type: (Object, Any) -> JsonElement
        pass


class GsonBuilder(object):
    def __init__(self, *args):
        # type: (*Any) -> None
        pass

    def addDeserializationExclusionStrategy(self, strategy):
        # type: (ExclusionStrategy) -> GsonBuilder
        pass

    def addSerializationExclusionStrategy(self, strategy):
        # type: (ExclusionStrategy) -> GsonBuilder
        pass

    def create(self):
        # type: () -> Gson
        pass

    def disableHtmlEscaping(self):
        # type: () -> GsonBuilder
        pass

    def disableInnerClassSerialization(self):
        # type: () -> GsonBuilder
        pass

    def enableComplexMapKeySerialization(self):
        # type: () -> GsonBuilder
        pass

    def excludeFieldsWithoutExposeAnnotation(self):
        # type: () -> GsonBuilder
        pass

    def excludeFieldsWithModifiers(self, *args):
        # type: (*int) -> GsonBuilder
        pass

    def generateNonExecutableJson(self):
        # type: () -> GsonBuilder
        pass

    def setLenient(self):
        # type: () -> GsonBuilder
        pass

    def registerTypeAdapter(self, type, typeAdapter):
        # type: (Any, Object) -> GsonBuilder
        pass

    def registerTypeAdapterFactory(self, factory):
        # type: (TypeAdapterFactory) -> GsonBuilder
        pass

    def registerTypeHierarchyAdapter(self, baseType, typeAdapter):
        # type: (Class, Object) -> GsonBuilder
        pass

    def serializeNulls(self):
        # type: () -> GsonBuilder
        pass

    def serializeSpecialFloatingPointValues(self):
        # type: () -> GsonBuilder
        pass

    def setDateFormat(self, *args):
        # type: (*Any) -> GsonBuilder
        pass

    def setExclusionStrategies(self, *args):
        # type: (*ExclusionStrategy) -> GsonBuilder
        pass

    def setFieldNamingPolicy(self, namingConvention):
        # type: (FieldNamingPolicy) -> GsonBuilder
        pass

    def setFieldNamingStrategy(self, fieldNamingStrategy):
        # type: (FieldNamingStrategy) -> GsonBuilder
        pass

    def setLongSerializationPolicy(self, policy):
        # type: (LongSerializationPolicy) -> GsonBuilder
        pass

    def setPrettyPrinting(self):
        # type: () -> GsonBuilder
        pass

    def setVersion(self, ignoreVersionsAfter):
        # type: (float) -> GsonBuilder
        pass


class JsonElement(object):
    def deepCopy(self):
        # type: () -> JsonElement
        pass

    def getAsBigDecimal(self):
        # type: () -> float
        raise NotImplementedError

    def getAsBigInteger(self):
        # type: () -> int
        raise NotImplementedError

    def getAsBoolean(self):
        # type: () -> bool
        raise NotImplementedError

    def getAsByte(self):
        # type: () -> Any
        raise NotImplementedError

    def getAsCharacter(self):
        # type: () -> str
        raise NotImplementedError

    def getAsDouble(self):
        # type: () -> float
        raise NotImplementedError

    def getAsFloat(self):
        # type: () -> float
        raise NotImplementedError

    def getAsInt(self):
        # type: () -> int
        raise NotImplementedError

    def getAsJsonArray(self):
        # type: () -> JsonArray
        pass

    def getAsJsonObject(self):
        # type: () -> JsonObject
        pass

    def getAsJsonPrimitive(self):
        # type: () -> JsonPrimitive
        pass

    def getAsJsonNull(self):
        # type: () -> JsonNull
        pass

    def getAsLong(self):
        # type: () -> long
        raise NotImplementedError

    def getAsNumber(self):
        # type: () -> Union[float, int, long]
        raise NotImplementedError

    def getAsShort(self):
        # type: () -> int
        raise NotImplementedError

    def isJsonArray(self):
        # type: () -> bool
        pass

    def isJsonObject(self):
        # type: () -> bool
        pass

    def isJsonPrimitive(self):
        # type: () -> bool
        pass

    def isJsonNull(self):
        # type: () -> bool
        pass


class JsonArray(JsonElement):
    def __init__(self, capacity=None):
        # type: (Optional[int]) -> None
        print(capacity)

    def add(self, arg):
        # type: (Any) -> None
        pass

    def addAll(self, array):
        # type: (JsonArray) -> None
        pass

    def contains(self, element):
        # type: (JsonElement) -> bool
        pass

    def get(self, i):
        # type: (int) -> JsonElement
        pass

    def getAsBigDecimal(self):
        # type: () -> float
        pass

    def getAsBigInteger(self):
        # type: () -> int
        pass

    def getAsBoolean(self):
        # type: () -> bool
        pass

    def getAsByte(self):
        # type: () -> Any
        pass

    def getAsCharacter(self):
        # type: () -> str
        pass

    def getAsDouble(self):
        # type: () -> float
        pass

    def getAsFloat(self):
        # type: () -> float
        pass

    def getAsInt(self):
        # type: () -> int
        pass

    def getAsLong(self):
        # type: () -> long
        pass

    def getAsNumber(self):
        # type: () -> Union[float, int, long]
        pass

    def getAsShort(self):
        # type: () -> int
        pass

    def iterator(self):
        # type: () -> Iterable[JsonElement]
        pass

    def remove(self, index):
        # type: (int) -> JsonElement
        pass

    def size(self):
        # type: () -> int
        pass


class JsonNull(JsonElement):
    def equals(self, other):
        # type: (Object) -> bool
        pass

    def getAsBigDecimal(self):
        # type: () -> float
        pass

    def getAsBigInteger(self):
        # type: () -> int
        pass

    def getAsBoolean(self):
        # type: () -> bool
        pass

    def getAsByte(self):
        # type: () -> Any
        pass

    def getAsCharacter(self):
        # type: () -> str
        pass

    def getAsDouble(self):
        # type: () -> float
        pass

    def getAsFloat(self):
        # type: () -> float
        pass

    def getAsInt(self):
        # type: () -> int
        pass

    def getAsLong(self):
        # type: () -> long
        pass

    def getAsNumber(self):
        # type: () -> Union[float, int, long]
        pass

    def getAsShort(self):
        # type: () -> int
        pass

    def hashCode(self):
        # type: () -> int
        pass


class JsonObject(JsonElement):
    def add(self, property, value):
        # type: (String, JsonElement) -> None
        pass

    def addProperty(self, property, value):
        # type: (String, Any) -> None
        pass

    def createJsonElement(self, value):
        # type: (Object) -> JsonElement
        pass

    def get(self, memberName):
        # type: (String) -> JsonElement
        pass

    def getAsBigDecimal(self):
        # type: () -> float
        pass

    def getAsBigInteger(self):
        # type: () -> int
        pass

    def getAsBoolean(self):
        # type: () -> bool
        pass

    def getAsByte(self):
        # type: () -> Any
        pass

    def getAsCharacter(self):
        # type: () -> str
        pass

    def getAsDouble(self):
        # type: () -> float
        pass

    def getAsFloat(self):
        # type: () -> float
        pass

    def getAsInt(self):
        # type: () -> int
        pass

    def getAsLong(self):
        # type: () -> long
        pass

    def getAsNumber(self):
        # type: () -> Union[float, int, long]
        pass

    def getAsShort(self):
        # type: () -> int
        pass

    def has(self, memberName):
        # type: (String) -> bool
        pass

    def remove(self, property):
        # type: (String) -> JsonElement
        pass

    def size(self):
        # type: () -> int
        pass


class JsonPrimitive(JsonElement):
    def __init__(self, arg):
        # type: (Any) -> None
        print(arg)

    def equals(self, other):
        # type: (Object) -> bool
        pass

    def getAsBigDecimal(self):
        # type: () -> float
        pass

    def getAsBigInteger(self):
        # type: () -> int
        pass

    def getAsBoolean(self):
        # type: () -> bool
        pass

    def getAsByte(self):
        # type: () -> Any
        pass

    def getAsCharacter(self):
        # type: () -> str
        pass

    def getAsDouble(self):
        # type: () -> float
        pass

    def getAsFloat(self):
        # type: () -> float
        pass

    def getAsInt(self):
        # type: () -> int
        pass

    def getAsLong(self):
        # type: () -> long
        pass

    def getAsNumber(self):
        # type: () -> Union[float, int, long]
        pass

    def getAsShort(self):
        # type: () -> int
        pass

    def hashCode(self):
        # type: () -> int
        pass

    def isBoolean(self):
        # type: () -> bool
        pass

    @staticmethod
    def isPrimitiveOrString(target):
        # type: (Object) -> bool
        pass

    @staticmethod
    def isIntegral(primitive):
        # type: (JsonPrimitive) -> bool
        pass

    def isString(self):
        # type: () -> bool
        pass


class LongSerializationPolicy(object):
    DEFAULT = None  # type: JsonElement
    STRING = None  # type: JsonElement

    def serialize(self, arg):
        # type: (long) -> JsonElement
        raise NotImplementedError


class TypeAdapter(object):
    def fromJson(self, arg):
        # type: (Union[Reader, String]) -> T
        pass

    def fromJsonTree(self, jsonTree):
        # type: (JsonElement) -> T
        pass

    def nullSafe(self):
        # type: () -> TypeAdapter
        pass

    def read(self, reader):
        # type: (JsonReader) -> T
        raise NotImplementedError

    def toJson(self, out, value):
        # type: (Writer, T) -> Optional[String]
        pass

    def toJsonTree(self, value):
        # type: (T) -> JsonElement
        pass

    def write(self, *args):
        raise NotImplementedError



from typing import Any

from java.lang import Object


class TypeToken(Object):
    @staticmethod
    def get(type):
        # type: (Any) -> TypeToken
        pass

    @staticmethod
    def getArray(componentType):
        # type: (Any) -> TypeToken
        pass

    @staticmethod
    def getParameterized(*args):
        # type: (Any) -> TypeToken
        pass



from __future__ import print_function

__all__ = ["SerialScriptModule"]

from typing import Any

from java.lang import Object


class SerialScriptModule(Object):
    def __init__(self):
        pass

    def setTrialExpired(self, arg):
        pass

    class PortManager(object):
        def __init__(self, *args):
            # type: (*Any) -> None
            pass

        def __enter__(self):
            print("Enter")

        def __exit__(self, exc_type, exc_val, exc_tb):
            print("Exit")

    class SerialConfigurator(object):
        def setBitRate(self, value):
            pass

        def setDataBits(self, value):
            pass

        def setFlowControl(self, value):
            pass

        def setHandshake(self, value):
            pass

        def setHardwareFlowControl(self, value):
            pass

        def setParity(self, value):
            pass

        def setStopBits(self, value):
            pass

    class SerialPortWrapper(object):
        def readBytes(self, port, numberOfBytes, timeout=5000):
            pass

        def readBytesAsString(
            self, port, numberOfBytes, timeout=5000, encoding="utf-8"
        ):
            pass

        def readLine(self, port, timeout=5000, encoding="utf-8", crlfRequired=False):
            pass

        def readUntil(self, port, delimiter, includeDelimiter, timeout=5000):
            pass

        def sendBreak(self, port, millis):
            pass

        def write(self, port, toWrite, encoding="utf-8"):
            pass

        def writeBytes(self, port, toWrite, timeout=5000):
            pass



from java.lang import Object, String
from java.util import EventObject


class PropertyChangeEvent(EventObject):
    def __init__(self, source, propertyName, oldValue, newValue):
        # type: (Object, String, Object, Object) -> None
        super(PropertyChangeEvent, self).__init__(source)
        self._newValue = newValue
        self._oldValue = oldValue
        self._propertyName = propertyName
        self._propagationId = Object()

    def getNewValue(self):
        # type: () -> Object
        return self._newValue

    def getOldValue(self):
        # type: () -> Object
        return self._oldValue

    def getPropagationId(self):
        # type: () -> Object
        return self._propagationId

    def setPropagationId(self, propagationId):
        # type: (Object) -> None
        self._propagationId = propagationId


class PropertyChangeListener(object):
    def propertyChange(self, evt):
        # type: (PropertyChangeEvent) -> None
        raise NotImplementedError



from typing import List

from java.lang import Enum


class ResolverStyle(Enum):
    LENIENT = None  # type: ResolverStyle
    SMART = None  # type: ResolverStyle
    STRICT = None  # type: ResolverStyle

    @staticmethod
    def values():
        # type: () -> List[ResolverStyle]
        pass



from typing import Any

from java.awt import AWTEvent, Component, Container
from java.lang import Object


class AncestorEvent(AWTEvent):
    ANCESTOR_ADDED = None  # type: int
    ANCESTOR_MOVED = None  # type: int
    ANCESTOR_REMOVED = None  # type: int

    def __init__(self, source, id, ancestor, ancestorParent):
        # type: (Component, int, Container, Container) -> None
        self._ancestor = ancestor
        self._ancestorParent = ancestorParent
        self._id = id
        self._source = source
        super(AncestorEvent, self).__init__(source, id)

    def getAncestor(self):
        # type: () -> Container
        return self._ancestor

    def getAncestorParent(self):
        # type: () -> Container
        return self._ancestorParent

    def getComponent(self):
        # type: () -> Component
        pass


class AncestorListener(object):
    def ancestorAdded(self, event):
        # type: (AncestorEvent) -> None
        raise NotImplementedError

    def ancestorMoved(self, event):
        # type: (AncestorEvent) -> None
        raise NotImplementedError

    def ancestorRemoved(self, event):
        # type: (AncestorEvent) -> None
        raise NotImplementedError



__all__ = ["Charset", "CharsetDecoder", "CharsetEncoder", "CoderResult"]

from typing import Dict, Optional, Set, Union

from java.lang import Object, String
from java.nio import ByteBuffer, CharBuffer
from java.util import Locale


class Charset(Object):
    def aliases(self):
        # type: () -> Set[String]
        pass

    @staticmethod
    def availableCharsets():
        # type: () -> Dict[String, Charset]
        pass

    def canEncode(self):
        # type: () -> bool
        pass

    def compareTo(self, that):
        # type: (Charset) -> int
        pass

    def contains(self, cs):
        # type: (Charset) -> bool
        pass

    def decode(self, bb):
        # type: (ByteBuffer) -> CharBuffer
        pass

    @staticmethod
    def defaultCharset():
        # type: () -> Charset
        pass

    def displayName(self, locale):
        # type: (Optional[Locale]) -> String
        pass

    def encode(self, arg):
        # type: (Union[Charset, String]) -> ByteBuffer
        pass

    @staticmethod
    def forName(charsetName):
        # type: (String) -> Charset
        pass

    def isRegistered(self):
        # type: () -> bool
        pass

    @staticmethod
    def isSupported(charsetName):
        # type: (String) -> bool
        pass

    def name(self):
        # type: () -> String
        pass

    def newDecoder(self):
        # type: () -> CharsetDecoder
        pass

    def newEncoder(self):
        # type: () -> CharsetEncoder
        pass


class CharsetDecoder(Object):
    def charset(self):
        # type: () -> Charset
        pass

    def decode(
        self,
        in_,  # type: ByteBuffer
        out=None,  # type: Optional[CharBuffer]
        endOfInput=None,  # type: Optional[bool]
    ):
        # type: (...) -> CharBuffer
        pass


class CharsetEncoder(Object):
    def charset(self):
        # type: () -> Charset
        pass

    def encode(
        self,
        in_,  # type: ByteBuffer
        out=None,  # type: Optional[ByteBuffer]
        endOfInput=None,  # type: Optional[bool]
    ):
        # type: (...) -> Union[ByteBuffer, CoderResult]
        pass


class CoderResult(Object):
    OVERFLOW = None  # type: CoderResult
    UNDERFLOW = None  # type: CoderResult

    def isError(self):
        # type: () -> bool
        pass

    def isMalformed(self):
        # type: () -> bool
        pass

    def isOverflow(self):
        # type: () -> bool
        pass

    def isUnderFlow(self):
        # type: () -> bool
        pass

    def isUnmappable(self):
        # type: () -> bool
        pass

    def length(self):
        # type: () -> int
        pass

    @staticmethod
    def malformedForLength(length):
        # type: (int) -> CoderResult
        pass

    def throwException(self):
        # type: () -> None
        pass

    @staticmethod
    def unmappableForLength(length):
        # type: (int) -> CoderResult
        pass



__all__ = ["TagPath"]

from typing import Union

from com.inductiveautomation.ignition.common import Path
from com.inductiveautomation.ignition.common.config import Property
from java.lang import String


class TagPath(Path):
    def compareTo(self, that):
        # type: (TagPath) -> int
        pass

    def getChildPath(self, arg):
        # type: (Union[Property, String]) -> TagPath
        pass

    def getItemName(self):
        # type: () -> String
        pass

    def getLastPathComponent(self):
        # type: () -> String
        pass

    def getParentPath(self):
        # type: () -> Path
        pass

    def getPathComponent(self, i):
        # type: (int) -> String
        pass

    def getPathLength(self):
        # type: () -> int
        pass

    def getProperty(self):
        # type: () -> Property
        pass

    def getSource(self):
        # type: () -> String
        pass

    def isAncestorOf(self, path):
        # type: (Path) -> bool
        pass

    def toStringFull(self):
        # type: () -> String
        pass

    def toStringPartial(self):
        # type: () -> String
        pass



__all__ = ["BaseStream", "Stream"]

from typing import TypeVar

from java.lang import AutoCloseable, Runnable
from java.util import Iterator, Spliterator
from java.util.function import Consumer

S = TypeVar("S")
T = TypeVar("T")


class BaseStream(AutoCloseable):
    def close(self):
        # type: () -> None
        raise NotImplementedError

    def isParallel(self):
        # type: () -> bool
        raise NotImplementedError

    def iterator(self):
        # type: () -> Iterator
        raise NotImplementedError

    def onClose(self, closeHandler):
        # type: (Runnable) -> S
        raise NotImplementedError

    def parallel(self):
        # type: () -> S
        raise NotImplementedError

    def sequential(self):
        # type: () -> S
        raise NotImplementedError

    def spliterator(self):
        # type: () -> Spliterator
        raise NotImplementedError

    def unordered(self):
        # type: () -> S
        raise NotImplementedError


class Stream(BaseStream):
    @staticmethod
    def builder():
        # type: () -> Stream.Builder
        pass

    def close(self):
        # type: () -> None
        pass

    def isParallel(self):
        # type: () -> bool
        pass

    def iterator(self):
        # type: () -> Iterator
        pass

    def onClose(self, closeHandler):
        # type: (Runnable) -> S
        pass

    def parallel(self):
        # type: () -> S
        pass

    def sequential(self):
        # type: () -> S
        pass

    def spliterator(self):
        # type: () -> Spliterator
        pass

    def unordered(self):
        # type: () -> S
        pass

    class Builder(Consumer):
        def accept(self, t):
            # type: (T) -> None
            raise NotImplementedError

        def add(self, t):
            # type: (T) -> Stream.Builder
            pass

        def build(self):
            raise NotImplementedError



__all__ = ["Logger", "Marker"]

from typing import Any, Iterable, Optional, Union

from java.lang import String


class Logger(object):
    ROOT_LOGGER_NAME = "ROOT"

    def debug(self, *args):
        # type: (Any) -> None
        pass

    def error(self, *args):
        # type: (Any) -> None
        pass

    def getName(self):
        # type: () -> String
        pass

    def info(self, *args):
        # type: (Any) -> None
        pass

    def isErrorEnabled(self, marker=None):
        # type: (Optional[Marker]) -> bool
        pass

    def isInfoEnabled(self, marker=None):
        # type: (Optional[Marker]) -> bool
        pass

    def isTraceEnabled(self, marker=None):
        # type: (Optional[Marker]) -> bool
        pass

    def isWarnEnabled(self, marker=None):
        # type: (Optional[Marker]) -> bool
        pass

    def trace(self, *args):
        # type: (Any) -> None
        pass

    def warn(self, *args):
        # type: (Any) -> None
        pass


class Marker(object):
    ANY_MARKER = "*"
    ANY_NON_NULL_MARKER = "+"

    def add(self, reference):
        # type: (Marker) -> None
        raise NotImplementedError

    def contains(self, arg):
        # type: (Union[Marker, String]) -> bool
        raise NotImplementedError

    def equals(self, o):
        # type: (Any) -> bool
        pass

    def getName(self):
        # type: () -> String
        pass

    def hashCode(self):
        # type: () -> int
        pass

    def hasReferences(self):
        # type: () -> bool
        pass

    def iterator(self):
        # type: () -> Iterable[Marker]
        pass

    def remove(self, reference):
        # type: (Marker) -> bool
        pass



"""BACnet enums."""

__all__ = ["DayOfWeek", "MaxApduLength", "MaxSegments", "Month"]

from enum import Enum


class DayOfWeek(Enum):
    """DayOfWeek enumerator."""

    MONDAY = 1
    TUESDAY = 2
    WEDNESDAY = 3
    THURSDAY = 4
    FRIDAY = 5
    SATURDAY = 6
    SUNDAY = 7
    UNSPECIFIED = 255


class MaxApduLength(Enum):
    """MaxApduLength enumerator."""

    UP_TO_50 = 0
    UP_TO_128 = 1
    UP_TO_206 = 2
    UP_TO_480 = 3
    UP_TO_1024 = 4
    UP_TO_1476 = 5


class MaxSegments(Enum):
    """MaxSegments enumerator."""

    UNSPECIFIED = 0
    UP_TO_2 = 1
    UP_TO_4 = 2
    UP_TO_8 = 3
    UP_TO_16 = 4
    UP_TO_32 = 5
    UP_TO_64 = 6
    MORE_THAN_64 = 7


class Month(Enum):
    """Month enumerator."""

    JANUARY = 1
    FEBRUARY = 2
    MARCH = 3
    APRIL = 4
    MAY = 5
    JUNE = 6
    JULY = 7
    AUGUST = 8
    SEPTEMBER = 9
    OCTOBER = 10
    NOVEMBER = 11
    DECEMBER = 12
    ODD_MONTHS = 13
    EVEN_MONTHS = 14
    UNSPECIFIED = 255



"""Print Functions.

The following functions allow you to send to a printer.
"""

from __future__ import print_function

__all__ = ["LANDSCAPE", "PORTRAIT", "createImage", "createPrintJob", "printToImage"]

from typing import Optional

from com.inductiveautomation.factorypmi.application.script.builtin import PrintUtilities
from java.awt import Component
from java.awt.image import BufferedImage
from java.lang import String

LANDSCAPE = 0
PORTRAIT = 1


def createImage(component):
    # type: (Component) -> BufferedImage
    """Takes a snapshot of a component and creates a Java BufferedImage
    out of it.

    You can use javax.imageio. to turn this into bytes that can be saved
    to a file or a BLOB field in a database.

    Args:
        component: The component to render.

    Returns:
        A java.awt.image.BufferedImage representing the component.
    """
    return PrintUtilities("app").createImage(component)


def createPrintJob(component):
    # type: (Component) -> PrintUtilities.JythonPrintJob
    """Provides a general printing facility for printing the contents of
    a window or component to a printer.

    The general workflow for this function is that you create the print
    job, set the options you'd like on it, and then call print() on the
    job. For printing reports or tables, use those components' dedicated
    print() functions.

    Args:
        component: The component that you'd like to print.

    Returns:
        A print job that can then be customized and started. To start
        the print job, use .print().
    """
    print(component)
    return PrintUtilities.JythonPrintJob()


def printToImage(component, filename=None):
    # type: (Component, Optional[String]) -> None
    """This function prints the given component (such as a graph,
    container, entire window, etc.) to an image file, and saves the file
    where ever the operating system deems appropriate.

    A filename and path may be provided to determine the name and
    location of the saved file.

    While not required, it is highly recommended to pass in a filename
    and path. The script may fail if the function attempts to save to a
    directory that the client does not have access rights to.

    Args:
        component: The component to render.
        filename: A filename to save the image as. Optional.
    """
    print(component, filename)



"""GUI Functions.

The following functions allow you to control windows and create popup
interfaces.
"""

from __future__ import print_function

__all__ = [
    "ACCL_CONSTANT",
    "ACCL_EASE",
    "ACCL_FAST_TO_SLOW",
    "ACCL_NONE",
    "ACCL_SLOW_TO_FAST",
    "COORD_DESIGNER",
    "COORD_SCREEN",
    "chooseColor",
    "closeDesktop",
    "color",
    "confirm",
    "convertPointToScreen",
    "createPopupMenu",
    "desktop",
    "errorBox",
    "findWindow",
    "getCurrentDesktop",
    "getDesktopHandles",
    "getOpenedWindowNames",
    "getOpenedWindows",
    "getParentWindow",
    "getQuality",
    "getScreenIndex",
    "getScreens",
    "getSibling",
    "getWindow",
    "getWindowNames",
    "inputBox",
    "isTouchscreenModeEnabled",
    "messageBox",
    "openDesktop",
    "openDiagnostics",
    "passwordBox",
    "setScreenIndex",
    "setTouchscreenModeEnabled",
    "showNumericKeypad",
    "showTouchscreenKeyboard",
    "transform",
    "warningBox",
]

from typing import Any, Callable, List, Optional, Tuple, Union

from com.inductiveautomation.factorypmi.application import FPMIWindow
from com.inductiveautomation.factorypmi.application.script.builtin import (
    WindowUtilities,
)
from java.awt import Color
from java.lang import String
from java.org.jdesktop.core.animation.timing import Animator
from java.util import EventObject
from javax.swing import (
    JComponent,
    JFrame,
    JLabel,
    JOptionPane,
    JPanel,
    JPopupMenu,
    JTextField,
)

# Constants
ACCL_NONE = 0
ACCL_CONSTANT = 1
ACCL_FAST_TO_SLOW = 2
ACCL_SLOW_TO_FAST = 3
ACCL_EASE = 4
COORD_SCREEN = 0
COORD_DESIGNER = 1


def chooseColor(initialColor, dialogTitle="Choose Color"):
    # type: (Color, Optional[String]) -> Color
    """Prompts the user to pick a color using the default color-chooser
    dialog box.

    Args:
        initialColor: A color to use as a starting point in the color
            choosing popup.
        dialogTitle: The title for the color choosing popup. Defaults to
            "Choose Color". Optional.

    Returns:
        The new color chosen by the user.
    """
    print(initialColor, dialogTitle)
    return Color()


def closeDesktop(handle):
    # type: (String) -> None
    """Allows you to close any of the open desktops associated with the
    current client.

    Args:
        handle: The handle for the desktop to close. The screen index
            cast as a string may be used instead of the handle. If
            omitted, this will default to the Primary Desktop.
            Alternatively, the handle "primary" can be used to refer to
            the Primary Desktop.
    """
    print(handle)


def color(*args):
    # type: (*Any) -> Color
    """Creates a new color object, either by parsing a string or by
    having the RGB[A] channels specified explicitly.

    Args:
        args: Variable-length argument list.

    Returns:
        The newly created color.
    """
    print(args)
    return Color(*args)


def confirm(message, title="Confirm", allowCancel=False):
    # type: (String, Optional[String], Optional[bool]) -> Optional[bool]
    """Displays a confirmation dialog box to the user with "Yes", "No"
    options, and a custom message.

    Args:
        message: The message to show in the confirmation dialog.
        title: The title for the confirmation dialog. Optional.
        allowCancel: Show a cancel button in the dialog. Optional.

    Returns:
        True if the user selected "Yes", False if the user
        selected "No", None if the user selected "Cancel".
    """
    options = ["Yes", "No"]

    if allowCancel:
        options.append("Cancel")

    choice = JOptionPane.showOptionDialog(
        None,
        message,
        title,
        JOptionPane.YES_NO_CANCEL_OPTION,
        JOptionPane.QUESTION_MESSAGE,
        None,
        options,
        options[0],
    )

    return (
        not bool(choice)
        if choice in [JOptionPane.YES_OPTION, JOptionPane.NO_OPTION]
        else None
    )


def convertPointToScreen(x, y, event):
    # type: (int, int, EventObject) -> Tuple[int, int]
    """Converts a pair of coordinates that are relative to
    the upper-left corner of some component to be relative to the
    upper-left corner of the entire screen.

    Args:
        x: The X-coordinate, relative to the component that fired the
            event.
        y: The Y-coordinate, relative to the component that fired the
            event.
        event: An event object for a component event.

    Returns:
        A tuple of (x,y) in screen coordinates.
    """
    print(x, y, event)
    return x, y


def createPopupMenu(itemNames, itemFunctions):
    # type: (List[String], List[Callable[..., Any]]) -> JPopupMenu
    """Creates a new popup menu, which can then be shown over a
    component on a mouse event.

    Args:
        itemNames: A list of names to create popup menu items with.
        itemFunctions: A list of functions to match up with the names.

    Returns:
        The javax.swing.JPopupMenu that was created.
    """
    print(itemNames, itemFunctions)
    return JPopupMenu()


def desktop(handle):
    # type: (String) -> WindowUtilities
    """Allows for invoking system.gui functions on a specific desktop.

    Args:
        handle: The handle for the desktop to use. The screen index cast
            as a string may be used instead of the handle. If omitted,
            this will default to the Primary Desktop. Alternatively, the
            handle "primary" can be used to refer to the primary
            desktop.

    Returns:
        A copy of system.gui that will be relative to the desktop named
        by the given handle.
    """
    print(handle)
    return WindowUtilities()


def errorBox(message, title="Error"):
    # type: (String, Optional[String]) -> None
    """Displays an error-style message box to the user.

    Args:
        message: The message to display in an error box.
        title: The title for the error box. Optional.
    """
    JOptionPane.showMessageDialog(None, message, title, JOptionPane.ERROR_MESSAGE)


def findWindow(path):
    # type: (String) -> List[FPMIWindow]
    """Finds and returns a list of windows with the given path.

    If the window is not open, an empty list will be returned. Useful
    for finding all instances of an open window that were opened with
    system.gui.openWindowInstance.

    Args:
        path: The path of the window to search for.

    Returns:
        A list of window objects. May be empty if window is not open, or
        have more than one entry if multiple windows are open.
    """
    print(path)
    return [FPMIWindow("Window")]


def getCurrentDesktop():
    # type: () -> String
    """Returns the handle of the desktop this function was called from.

    Commonly used with the system.gui.desktop and system.nav.desktop
    functions.

    Returns:
        The handle of the current desktop.
    """
    return "primary"


def getDesktopHandles():
    # type: () -> List[String]
    """Gets a list of all secondary handles of the open desktops
    associated with the current client.

    In this case, secondary means any desktop frame opened by the
    original client frame.

    Example:
        If the original client opened 2 new frames ('left client' and
        'right client'), then this function would return ['left client',
        'right client'].

    Returns:
        A list of window handles of all secondary Desktop frames.
    """
    return ["left client", "right client"]


def getOpenedWindowNames():
    # type: () -> Tuple[String, ...]
    """Finds all of the currently open windows and returns a tuple of
    their paths.

    Returns:
        A tuple of strings, representing the path of each window that is
        open.
    """
    return "window_1", "window_2", "window_n"


def getOpenedWindows():
    # type: () -> Tuple[FPMIWindow, ...]
    """Finds all of the currently open windows, returning a tuple of
    references to them.

    Returns:
         A tuple of the opened windows. Not their names, but the actual
         window objects themselves.
    """
    return FPMIWindow("Main Window"), FPMIWindow("Other Window")


def getParentWindow(event):
    # type: (EventObject) -> FPMIWindow
    """Finds the parent (enclosing) window for the component that fired
    an event, returning a reference to it.

    Args:
        A component event object.

    Returns:
        The window that contains the component that fired the event.
    """
    print(event)
    return FPMIWindow("Parent Window")


def getQuality(component, propertyName):
    # type: (JComponent, String) -> int
    """Returns the data quality for the property of the given component
    as an integer.

    This function can be used to check the quality of a Tag binding on a
    component in the middle of the script so that alternative actions
    can be taken in the event of device disconnections.

    Args:
        component: The component whose property is being checked.
        propertyName: The name of the property as a string value.

    Returns:
        The data quality of the given property as an integer.
    """
    print(component, propertyName)
    return 192


def getScreenIndex():
    # type: () -> int
    """Returns the returns an integer value representing the current
    screen index based on the screen from which this function was
    called.

    Returns:
        The screen from which the function was called.
    """
    return 0


def getScreens():
    # type: () -> List[Tuple[String, int, int]]
    """Get a list of all the monitors on the computer this client is
    open on.

    Use with system.gui.setScreenIndex() to move the client.

    Returns:
        A sequence of tuples of the form (index, width, height) for each
        screen device (monitor) available.
    """
    return [("primary", 1440, 900), ("secondary", 1920, 1080)]


def getSibling(event, name):
    # type: (EventObject, String) -> FPMIWindow
    """Given a component event object, looks up a sibling component.

    Shortcut for event.source.parent.getComponent("siblingName"). If no
    such sibling is found, the special value None is returned.

    Args:
        event: A component event object.
        name: The name of the sibling component.

    Returns:
        The sibling component itself.
    """
    print(event, name)
    return FPMIWindow("Sibling")


def getWindow(name):
    # type: (String) -> FPMIWindow
    """Finds a reference to an open window with the given name.

    Throws a ValueError if the named window is not open or not found.

    Args:
        name: The path to the window to field.

    Returns:
        A reference to the window, if it was open.
    """
    print(name)
    return FPMIWindow("Main Window")


def getWindowNames():
    # type: () -> Tuple[String, ...]
    """Returns a list of the paths of all windows in the current
    project, sorted alphabetically.

    Returns:
        A tuple of strings, representing the path of each window defined
        in the current project.
    """
    return "Main Window", "Main Window 1", "Main Window 2"


def inputBox(message, defaultText=""):
    # type: (String, String) -> Optional[String]
    """Opens up a popup input dialog box.

    This dialog box will show a prompt message, and allow the user to
    type in a string. When the user is done, they can press "OK" or
    "Cancel". If OK is pressed, this function will return with the value
    that they typed in. If Cancel is pressed, this function will return
    the value None.

    Args:
        message: The message to display for the input box. Will accept
            HTML formatting.
        defaultText: The default text to initialize the input box with.
            Optional.

    Returns:
        The string value that was entered in the input box.
    """
    options = ["OK", "Cancel"]

    panel = JPanel()
    label = JLabel("{}: ".format(message))
    panel.add(label)
    text_field = JTextField(25)
    text_field.setText(defaultText)
    panel.add(text_field)

    choice = JOptionPane.showOptionDialog(
        None,
        panel,
        "Input",
        JOptionPane.OK_CANCEL_OPTION,
        JOptionPane.QUESTION_MESSAGE,
        None,
        options,
        options[0],
    )

    return text_field.getText() if choice == JOptionPane.OK_OPTION else None


def isTouchscreenModeEnabled():
    # type: () -> bool
    """Checks whether or not the running client's touchscreen mode is
    currently enabled.

    Returns:
         True if the Client currently has Touch Screen mode activated.
    """
    return False


def messageBox(message, title="Information"):
    # type: (String, String) -> None
    """Displays an informational-style message popup box to the user.

    Args:
        message: The message to display. Will accept HTML formatting.
        title: The title for the message box. Optional.
    """
    JOptionPane.showMessageDialog(None, message, title, JOptionPane.INFORMATION_MESSAGE)


def openDesktop(
    screen=0,  # type: int
    handle=None,  # type: Optional[String]
    title=None,  # type: Optional[String]
    width=None,  # type: Optional[int]
    height=None,  # type: Optional[int]
    x=0,  # type: int
    y=0,  # type: int
    windows=None,  # type: Optional[List[String]]
):
    # type: (...) -> JFrame
    """Creates an additional Desktop in a new frame.

    Args:
        screen: The screen index of which screen to place the new frame
            on. If omitted, screen 0 will be used. Optional.
        handle: A name for the desktop. If omitted, the screen index
            will be used. Optional.
        title: The title for the new frame. If omitted, the index handle
            will be used. If the handle and title are omitted, the
            screen index will be used. Optional.
        width: The width for the new Desktop's frame. If omitted, frame
            will become maximized on the specified monitor. Optional.
        height: The width for the new desktop's frame. If omitted, frame
            will become maximized on the specified monitor. Optional.
        x: The X coordinate for the new desktop's frame. Only used if
            both width and height are specified. If omitted, defaults to
            0. Optional.
        y: The Y coordinate for the new desktop's frame. Only used if
            both width and height are specified. If omitted, defaults to
            0. Optional.
        windows: A list of window paths to open in the new Desktop
            frame. If omitted, the desktop will open without any opened
            windows. Optional.

    Returns:
        A reference to the new Desktop frame object.
    """
    print(screen, handle, title, width, height, x, y, windows)
    return JFrame()


def openDiagnostics():
    # type: () -> None
    """Opens the client runtime diagnostics window, which provides
    information regarding performance, logging, active threads,
    connection status, and the console.

    This provides an opportunity to open the diagnostics window in
    situations where the menu bar in the client is hidden, and the
    keyboard shortcut can not be used.
    """
    pass


def passwordBox(
    message,  # type:String
    title="Password",  # type: String
    echoChar="*",  # type: String
):
    # type: (...) -> Optional[String]
    """Pops up a special input box that uses a password field, so the
    text isn't echoed back in clear-text to the user.

    Returns the text they entered, or None if they canceled the dialog
    box.

    Args:
        message: The message for the password prompt. Will accept HTML
            formatting.
        title: A title for the password prompt. Optional.
        echoChar: A custom echo character. Defaults to: *. Optional.

    Returns:
        The password that was entered, or None if the prompt was
        canceled.
    """
    print(message, title, echoChar)
    return "password"


def setScreenIndex(index):
    # type: (int) -> None
    """Moves an open client to a specific monitor.

    Use with system.gui.getScreens() to identify monitors before moving.

    Args:
        index: The new monitor index for this client to move to. 0
            based.
    """
    print(index)


def setTouchscreenModeEnabled(enabled):
    # type: (bool) -> None
    """Alters a running Client's Touch Screen mode on the fly.

    Args:
        enabled: The new value for Touch Screen mode being enabled.
    """
    print(enabled)


def showNumericKeypad(
    initialValue,  # type: Union[float, int]
    fontSize=None,  # type: Optional[int]
    usePasswordMode=False,  # type: bool
):
    # type: (...) -> Union[float, int]
    """Displays a modal on-screen numeric keypad, allowing for arbitrary
    numeric entry using the mouse, or a finger on a touchscreen monitor.

    Returns the number that the user entered.

    Args:
        initialValue: The value to start the on-screen keypad with.
        fontSize: The font size to display in the keypad. Optional.
        usePasswordMode: If True, display a * for each digit. Optional.

    Returns:
        The value that was entered in the keypad.
    """
    print(initialValue, fontSize, usePasswordMode)
    return 43


def showTouchscreenKeyboard(initialText, fontSize=None, passwordMode=False):
    # type: (String, Optional[int], Optional[bool]) -> String
    """Displays a modal on-screen keyboard, allowing for arbitrary text
    entry using the mouse, or a finger on a touchscreen monitor.

    Returns the text that the user entered.

    Args:
        initialText: The text to start the on-screen keyboard with.
        fontSize: The font size to display in the keypad. Optional.
        passwordMode: True to activate password mode, where the text
            entered isn't echoed back clear-text. Optional.

    Returns:
        The text that was entered in the on-screen keyboard.
    """
    print(initialText, fontSize, passwordMode)
    return ""


def transform(
    component,  # type: JComponent
    newX=None,  # type: Optional[int]
    newY=None,  # type: Optional[int]
    newWidth=None,  # type: Optional[int]
    newHeight=None,  # type: Optional[int]
    duration=0,  # type: int
    callback=None,  # type: Optional[Callable[..., Any]]
    framesPerSecond=60,  # type: int
    acceleration=None,  # type: Optional[int]
    coordSpace=None,  # type: Optional[int]
):
    # type: (...) -> Animator
    """Sets a component's position and size at runtime.

    Additional arguments for the duration, framesPerSecond, and
    acceleration of the operation exist for animation. An optional
    callback argument will be executed when the transformation is
    complete.

    Note:
        The transformation is performed in Designer coordinate space on
        components which are centered or have more than two anchors.

    Args:
        component: The component to move or resize.
        newX: An optional x-coordinate to move to, relative to the
            upper-left corner of the component's parent container.
        newY: An optional y-coordinate to move to, relative to the
            upper-left corner of the component's parent container.
        newWidth: An optional width for the component.
        newHeight: An optional height for the component.
        duration: An optional duration over which the transformation
            will take place. If omitted or 0, the transform will take
            place immediately.
        callback: An optional function to be called when the
            transformation is complete.
        framesPerSecond: An optional frame rate argument which dictates
            how often the transformation updates over the given
            duration. The default is 60 frames per second.
        acceleration: An optional modifier to the acceleration of the
            transformation over the given duration. See system.gui
            constants for valid arguments.
        coordSpace: The coordinate space to use. When the default Screen
            Coordinates are used, the given size and position are
            absolute, as they appear in the client at runtime. When
            Designer Coordinates are used, the given size and position
            are pre-runtime adjusted values, as they would appear in the
            Designer. See system.gui constants for valid arguments.

    Returns:
        An object that contains pause(), resume(), and cancel() methods,
        allowing for a script to interrupt the animation.
    """
    print(
        component,
        newX,
        newY,
        newWidth,
        newHeight,
        duration,
        callback,
        framesPerSecond,
        acceleration,
        coordSpace,
    )
    return Animator()


def warningBox(message, title="Warning"):
    # type: (String, String) -> None
    """Displays a message to the user in a warning style popup dialog.

    Args:
        message: The message to display in the warning box. Will accept
            HTML formatting.
        title: The title for the warning box. Optional.
    """
    JOptionPane.showMessageDialog(None, message, title, JOptionPane.WARNING_MESSAGE)



__all__ = ["MessageInterface", "MessageReceiver", "UIResponse"]

from typing import List

from com.inductiveautomation.ignition.common.functional import FragileRunnable
from java.lang import Object, String
from java.util import Locale
from java.util.function import Consumer


class MessageInterface(object):
    def addMessageReceiver(self, protocol, rcv):
        raise NotImplementedError

    def sendCall(self, protocol, scope, msg):
        raise NotImplementedError

    def sendMessage(self, protocol, scope, msg):
        raise NotImplementedError


class MessageReceiver(object):
    def receiveCall(self, msg):
        raise NotImplementedError


class UIResponse(Object):
    errors = []  # type: List[String]
    infos = []  # type: List[String]
    locale = None  # type: Locale
    warns = []  # type: List[String]

    def __init__(self, locale):
        # type: (Locale) -> None
        self.locale = locale

    def attempt(self, method):
        # type: (FragileRunnable) -> None
        pass

    def error(self, message, *args):
        # type: (String, *Object) -> None
        pass

    def getErrors(self):
        # type: () -> List[String]
        pass

    def getInfos(self):
        # type: () -> List[String]
        pass

    def getLocale(self):
        # type: () -> Locale
        return self.locale

    def getWarns(self):
        # type: () -> List[String]
        pass

    def info(self, message, *args):
        # type: (String, *Object) -> None
        pass

    def warn(self, message, *args):
        # type: (String, *Object) -> None
        pass

    @staticmethod
    def wrap(locale, fx):
        # type: (Locale, Consumer) -> UIResponse
        pass



__all__ = [
    "AbstractScheduleModel",
    "BasicScheduleModel",
    "CompositeScheduleModel",
    "HolidayModel",
    "ScheduleAdjustment",
]

from com.palantir.ptoss.cinch.core import DefaultBindableModel
from java.lang import Object, String
from java.util import Date


class AbstractScheduleModel(DefaultBindableModel):
    def getScheduleForDay(self, cal):
        pass

    def getType(self):
        pass

    def isObserveHolidays(self):
        pass

    def setObserveHolidays(self, observeHolidays):
        pass


class BasicScheduleModel(AbstractScheduleModel):
    def getAllDayTime(self):
        pass

    def getFridayTime(self):
        pass

    def getMondayTime(self):
        pass

    def getRepeat(self):
        pass

    def getRepeatOff(self):
        pass

    def getRepeatOn(self):
        pass

    def getSaturdayTime(self):
        pass

    def getStartingAt(self):
        pass

    def getSundayTime(self):
        pass

    def getThursdayTime(self):
        pass

    def getTuesdayTime(self):
        pass

    def getWednesdayTime(self):
        pass

    def getWeekDayTime(self):
        pass

    def isAllDays(self):
        pass

    def isFriday(self):
        pass

    def isMonday(self):
        pass

    def isRepeating(self):
        pass

    def isSaturday(self):
        pass

    def isSunday(self):
        pass

    def isThursday(self):
        pass

    def isTuesday(self):
        pass

    def isUseDays(self):
        pass

    def isWednesday(self):
        pass

    def isWeekDays(self):
        pass

    def set(self, that):
        pass

    def setAllDays(self, allDays):
        pass

    def setAllDayTime(self, allDayTime):
        pass

    def setFriday(self, friday):
        pass

    def setFridayTime(self, fridayTime):
        pass

    def setMonday(self, monday):
        pass

    def setMondayTime(self, mondayTime):
        pass

    def setRepeat(self, repeat):
        pass

    def setRepeatOff(self, repeatOff):
        pass

    def setRepeatOn(self, repeatOn):
        pass

    def setSaturday(self, saturday):
        pass

    def setSaturdayTime(self, saturdayTime):
        pass

    def setStartingAt(self, startingAt):
        pass

    def setSunday(self, sunday):
        pass

    def setSundayTime(self, sundayTime):
        pass

    def setThursday(self, thursday):
        pass

    def setThursdayTime(self, thursdayTime):
        pass

    def setTuesday(self, tuesday):
        pass

    def setTuesdayTime(self, tuesdayTime):
        pass

    def setWednesday(self, wednesday):
        pass

    def setWednesdayTime(self, wednesdayTime):
        pass

    def setWeekDays(self, weekDays):
        pass

    def setWeekDayTime(self, weekDayTime):
        pass


class CompositeScheduleModel(AbstractScheduleModel):
    def getModels(self):
        pass


class HolidayModel(DefaultBindableModel):
    date = None  # type: Date
    name = None  # type: String
    repeatAnnually = None  # type: bool

    def __init__(self, name, date, repeatAnnually):
        # type: (String, Date, bool) -> None
        self.name = name
        self.date = date
        self.repeatAnnually = repeatAnnually

    def getDate(self):
        # type: () -> Date
        return self.date

    def getName(self):
        # type: () -> String
        return self.name

    def isRepeatAnnually(self):
        # type: () -> bool
        return self.repeatAnnually

    def set(self, that):
        # type: (HolidayModel) -> None
        pass

    def setDate(self, date):
        # type: (Date) -> None
        self.date = date

    def setName(self, name):
        # type: (String) -> None
        self.name = name

    def setRepeatAnnually(self, repeatAnnually):
        # type: (bool) -> None
        self.repeatAnnually = repeatAnnually


class ScheduleAdjustment(DefaultBindableModel):
    start = None  # type: Date
    end = None  # type: Date
    available = None  # type: bool
    note = None  # type: String

    def __init__(self, start, end, available, note):
        # type: (Date, Date, bool, String) -> None
        self.start = start
        self.end = end
        self.available = available
        self.note = note

    def contains(self, timestamp):
        # type: (long) -> bool
        pass

    def getEnd(self):
        # type: () -> Date
        return self.end

    def getNote(self):
        # type: () -> String
        return self.note

    def getStart(self):
        # type: () -> Date
        return self.start

    def isAvailable(self):
        # type: () -> bool
        return self.available

    def setAvailable(self, available):
        # type: (bool) -> None
        self.available = available

    def setEnd(self, end):
        # type: (Date) -> None
        self.end = end

    def setNote(self, note):
        # type: (String) -> None
        self.note = note

    def setStart(self, start):
        # type: (Date) -> None
        self.start = start



from __future__ import print_function

__all__ = ["AbstractOPCUtilities", "DatasetUtilities", "SProcCall", "SystemUtilities"]

from typing import Any, Dict, List, Optional, Tuple, Union

from com.inductiveautomation.ignition.common import BasicDataset, Dataset
from com.inductiveautomation.ignition.common.model.values import QualityCode
from com.inductiveautomation.ignition.common.opc import BrowseElementType
from com.inductiveautomation.ignition.common.script.abc import AbstractJythonSequence
from com.inductiveautomation.ignition.common.script.message import Request
from java.lang import Class
from java.lang import Exception as JavaException
from java.lang import Object, String
from java.util import Locale
from org.python.core import PyFunction, PyObject
from org.slf4j import Logger


class AbstractOPCUtilities(Object):
    def browseServer(self, opcServer, nodeId):
        # type: (String, String) -> List[AbstractOPCUtilities.PyOPCTag]
        return [
            AbstractOPCUtilities.PyOPCTag(opcServer, nodeId, "", BrowseElementType())
        ]

    def getServers(self):
        pass

    def getServerState(self, opcServer):
        pass

    def isServerEnabled(self, serverName):
        pass

    def readValue(self, opcServer, itemPath):
        pass

    def readValues(self, opcServer, itemPaths):
        pass

    def setServerEnabled(self, serverName, enabled):
        pass

    def writeValue(self, *args, **kwargs):
        pass

    def writeValues(self, *args, **kwargs):
        pass

    class PyOPCTag(PyObject):
        displayName = None  # type: String
        elementType = None  # type: BrowseElementType
        nodeId = None  # type: String
        serverName = None  # type: String

        def __init__(self, serverName, nodeId, displayName, elementType):
            # type: (String, String, String, BrowseElementType) -> None
            self.serverName = serverName
            self.nodeId = nodeId
            self.displayName = displayName
            self.elementType = elementType
            super(AbstractOPCUtilities.PyOPCTag, self).__init__()

        def __findattr_ex__(self, name):
            # type: (String) -> PyObject
            pass

        def getDisplayName(self):
            # type: () -> String
            return self.displayName

        def getElementType(self):
            # type: () -> BrowseElementType
            return self.elementType

        def getNodeId(self):
            # type: () -> String
            return self.nodeId

        def getServerName(self):
            # type: () -> String
            return self.serverName


class DatasetUtilities(Object):
    @staticmethod
    def addColumn(*args):
        pass

    @staticmethod
    def addRow(*args):
        pass

    @staticmethod
    def addRows(*args):
        pass

    @staticmethod
    def appendDataset(ds1, ds2):
        pass

    @staticmethod
    def clearDataset(ds):
        pass

    @staticmethod
    def dataSetToExcel(headerRow, datasets):
        pass

    @staticmethod
    def dataSetToExcelBytes(headerRow, objects, nullsEmpty, sheetNames):
        pass

    @staticmethod
    def dataSetToExcelStreaming(headerRow, objects, out, nullsEmpty):
        pass

    @staticmethod
    def dataSetToHTML(headerRow, ds, title):
        pass

    @staticmethod
    def dataSetToHTMLStreaming(headerRow, ds, title, fw):
        pass

    @staticmethod
    def deleteRow(ds, row):
        pass

    @staticmethod
    def deleteRows(ds, rows):
        pass

    @staticmethod
    def filterColumns(dataset, columns):
        pass

    @staticmethod
    def formatDates(dataset, format, locale=Locale.US):
        pass

    @staticmethod
    def fromCSV(csv):
        pass

    @staticmethod
    def fromCSVJava(csv):
        pass

    @staticmethod
    def getColumnHeaders(ds):
        pass

    @staticmethod
    def insertColumn(*args):
        pass

    @staticmethod
    def insertRow(*args):
        pass

    @staticmethod
    def setValue(*args):
        pass

    @staticmethod
    def sort(
        ds,  # type: BasicDataset
        keyColumn,  # type: Union[int, String]
        ascending=None,  # type: Optional[bool]
        naturalOrdering=None,  # type: Optional[bool]
    ):
        # type: (...) -> BasicDataset
        pass

    @staticmethod
    def toCSV(*args, **kwargs):
        pass

    @staticmethod
    def toCSVJava(ds, showHeaders, forExport, localized=False):
        pass

    @staticmethod
    def toCSVJavaStreaming(ds, showHeaders, forExport, sw, localized):
        pass

    @staticmethod
    def toDataSet(*args):
        pass

    @staticmethod
    def toExcel(*args, **kwargs):
        pass

    @staticmethod
    def toJSONObject(data):
        pass

    @staticmethod
    def toPyDataSet(dataset):
        # type: (Dataset) -> PyDataSet
        pass

    @staticmethod
    def updateRow(ds, row, changes):
        pass

    class PyDataSet(Dataset, AbstractJythonSequence):
        def __init__(self, ds=None):
            # type: (Optional[Dataset]) -> None
            pass

        def getColumnCount(self):
            # type: () -> int
            """Returns the number of columns in the dataset.

            Returns:
                The number of columns in the dataset.
            """
            pass

        def getColumnIndex(self, colName):
            # type: (String) -> int
            """Returns the index of the column with the name colName.

            Args:
                colName: The name of the column.

            Returns:
                The index of the column with the name colName.
            """
            pass

        def getColumnName(self, col):
            # type: (int) -> String
            """Returns the name of the column at the index colIndex.

            Args:
                col: The column number. Zero-indexed.

            Returns:
                The name of the column at the index colIndex.
            """
            pass

        def getColumnNames(self):
            # type: () -> List[String]
            """Returns a list with the names of all the columns.

            Returns:
                A list with the names of all the columns.
            """
            pass

        def getColumnType(self, col):
            # type: (int) -> Class
            """Returns the type of the column at the index.

            Args:
                col: The column number. Zero-indexed.

            Returns:
                The type of the column at the index.
            """
            pass

        def getColumnTypes(self):
            # type: () -> List[Class]
            """Returns a list with the types of all the columns.

            Returns:
                A list with the types of all the columns.
            """
            pass

        def getPrimitiveValueAt(self, row, col):
            # type: (int, int) -> float
            """If the given column is a numeric type or a Date, then the
            value will be returned as a double.

            Args:
                row: The row index. Zero-based index.
                col: The column index. Zero-based index.

            Raises:
                IllegalArgumentException: if the value at row, col is
                    not a number or Date.
                UnsupportedOperationException: If the Dataset
                    implementation declines to implement this operation.
            """
            pass

        def getQualityAt(self, row, col):
            # type: (int, int) -> QualityCode
            """Returns the quality of the value at the given location.

            Args:
                row: The row index. Zero-based index.
                col: The column index. Zero-based index.

            Raises:
                ArrayIndexOutOfBoundsException: If the given row, col is
                    out of range and hasQualityData() returns true.
            """
            pass

        def getRowCount(self):
            # type: () -> int
            """Returns the number of rows in the dataset.

            Returns:
                The number of rows in the dataset.
            """
            pass

        def getValueAt(self, row, col):
            # type: (int, Union[int, String]) -> Any
            """Returns the value at the specified row index and column
            name or index.

            Args:
                row: The row number. Zero-indexed.
                col: The column number (zero-indexed) or name.

            Returns:
                The value found at the row and column.
            """
            pass

        def setData(self, data):
            # type: (Dataset) -> None
            """Used for serialization only."""
            pass


class SProcCall(Object):
    callFinished = False  # type: bool
    datasource = ""  # type: String
    params = None  # type: Dict[SProcCall.SProcArgKey, SProcCall.SProcArg]
    procedureName = None  # type: String
    resultset = None  # type: Dataset
    returnParam = None  # type: SProcCall.SProcArg
    skipAudit = None  # type: bool
    txId = None  # type: String
    updateCount = None  # type: int

    def getDataSource(self):
        # type: () -> String
        pass

    def getOutParamValue(self, param):
        # type: (Union[int, String]) -> Any
        print(self, param)
        return 0

    def getProcedureName(self):
        # type: () -> String
        pass

    def getResultSet(self):
        # type: () -> BasicDataset
        print(self)
        return BasicDataset()

    def getReturnValue(self):
        # type: () -> Any
        print(self)
        return 0

    def getTxId(self):
        # type: () -> String
        return "transaction_id"

    def getUpdateCount(self):
        # type: () -> int
        print(self)
        return 1

    def isSkipAudit(self):
        # type: () -> bool
        print(self)
        return False

    def registerInParam(self, param, typeCode, value):
        # type: (Union[int, String], int, Any) -> None
        print(self, param, typeCode, value)

    def registerOutParam(self, param, typeCode):
        # type: (Union[int, String], int) -> None
        print(self, param, typeCode)

    def registerReturnParam(self, typeCode):
        # type: (int) -> None
        print(self, typeCode)

    def setSkipAudit(self, skipAudit):
        # type: (bool) -> None
        pass

    def setTxId(self, txId):
        # type: (String) -> None
        pass

    class SProcArg(Object):
        outParam = False  # type: bool
        inParam = False  # type: bool
        paramType = None  # type: int
        value = None  # type: Object

        def getParamType(self):
            # type: () -> int
            pass

        def getValue(self):
            # type: () -> Any
            pass

        def isInParam(self):
            # type: () -> bool
            pass

        def isOutParam(self):
            # type: () -> bool
            pass

        def setParamType(self, paramType):
            # type: (int) -> None
            pass

        def setValue(self, value):
            # type: (Object) -> None
            pass

    class SProcArgKey(Object):
        index = -1  # type: int
        name = None  # type: String

        def getParamIndex(self):
            # type: () -> int
            pass

        def getParamName(self):
            # type: () -> String
            pass

        def isNamedParam(self):
            # type: () -> bool
            pass


class SystemUtilities(Object):
    @staticmethod
    def logger(loggerName):
        # type: (String) -> Logger
        pass

    @staticmethod
    def parseTranslateArguments(*args, **kwargs):
        # type: (*PyObject, **String) -> Tuple[String, String, bool]
        pass

    class RequestImpl(Object, Request):
        timeout = None  # type: int

        def __init__(self, timeout):
            # type: (int) -> None
            self.timeout = timeout

        def cancel(self):
            # type: () -> None
            pass

        def checkTimeout(self):
            # type: () -> None
            pass

        def finishExceptionally(self, e):
            # type: (JavaException) -> None
            pass

        def finishSuccessfully(self, value):
            # type: (Object) -> None
            pass

        def get(self):
            # type: () -> Object
            pass

        def getError(self):
            # type: () -> JavaException
            pass

        def getLongId(self):
            # type: () -> long
            pass

        def onError(self, func):
            # type: (PyFunction) -> None
            pass

        def onSuccess(self, func):
            # type: (PyFunction) -> None
            pass



__all__ = ["TagPathParser"]

from typing import Any, List

from com.inductiveautomation.ignition.common.tags.model import TagPath
from java.lang import Object, String


class TagPathParser(Object):
    PARENT_RELATIVE = None  # type: String
    PATH_SEPARATOR = None  # type: String
    PROPERTY_SEPARATOR = None  # type: String
    RELATIVE_DIR_UP = None  # type: String
    ROOT_RELATIVE = None  # type: String

    @staticmethod
    def chopPath(string):
        # type: (String) -> List[String]
        pass

    @staticmethod
    def derelativize(tagPath, relativeRoot):
        # type: (TagPath, TagPath) -> TagPath
        pass

    @staticmethod
    def isRelativePath(path):
        # type: (TagPath) -> bool
        pass

    @staticmethod
    def parse(*args):
        # type: (*Any) -> TagPath
        pass

    @staticmethod
    def parseSafe(*args):
        # type: (*Any) -> TagPath
        pass

    @staticmethod
    def relativize(newPath, relativeTo):
        # type: (TagPath, TagPath) -> TagPath
        pass



__all__ = [
    "ChronoUnit",
    "Temporal",
    "TemporalAccessor",
    "TemporalAdjuster",
    "TemporalField",
    "TemporalQuery",
    "TemporalUnit",
    "ValueRange",
]

from typing import Any, Dict, List, Optional, TypeVar

from java.lang import Enum, Object, String
from java.time import Duration
from java.time.format import ResolverStyle
from java.util import Locale

R = TypeVar("R")


class TemporalAccessor(object):
    def get(self, field):
        # type: (TemporalField) -> int
        pass

    def getLong(self, field):
        # type: (TemporalField) -> long
        pass

    def isSupported(self, field):
        # type: (TemporalField) -> bool
        pass

    def query(self, query):
        # type: (TemporalQuery) -> R
        pass

    def range(self, field):
        # type: (TemporalField) -> ValueRange
        pass


class TemporalAdjuster(object):
    def adjustInto(self, temporal):
        # type: (Temporal) -> Temporal
        raise NotImplementedError


class TemporalField(object):
    def adjustInto(self, temporal, newValue):
        # type: (R, long) -> Temporal
        raise NotImplementedError

    def getBaseUnit(self):
        # type: () -> TemporalUnit
        raise NotImplementedError

    def getDisplayName(self, locale):
        # type: (Locale) -> String
        pass

    def getFrom(self, temporal):
        # type: (TemporalAccessor) -> long
        raise NotImplementedError

    def getRangeUnit(self):
        # type: () -> TemporalUnit
        raise NotImplementedError

    def isDateBased(self):
        # type: () -> bool
        raise NotImplementedError

    def isSupportedBy(self, temporal):
        # type: (TemporalAccessor) -> bool
        raise NotImplementedError

    def isTimeBased(self):
        # type: () -> bool
        raise NotImplementedError

    def range(self):
        # type: () -> ValueRange
        raise NotImplementedError

    def rangeRefinedBy(self, temporal):
        # type: (TemporalAccessor) -> ValueRange
        raise NotImplementedError

    def resolve(
        self,
        fieldValues,  # type: Dict[TemporalField, long]
        partialTemporal,  # type: TemporalAccessor
        resolverStyle,  # type: ResolverStyle
    ):
        # type: (...) -> TemporalAccessor
        pass

    def toString(self):
        # type: () -> String
        raise NotImplementedError


class TemporalQuery(object):
    def queryFrom(self, temporal):
        # type: (TemporalAccessor) -> R
        raise NotImplementedError


class TemporalUnit(object):
    def addTo(self, temporal, amount):
        # type: (Temporal, long) -> Temporal
        raise NotImplementedError

    def between(self, temporal1Inclusive, temporal2Exclusive):
        # type: (Temporal, Temporal) -> Temporal
        raise NotImplementedError

    def isSupportedBy(self, temporal):
        # type: (Temporal) -> bool
        pass

    def getDuration(self):
        # type: () -> Duration
        raise NotImplementedError

    def isDateBased(self):
        # type: () -> bool
        raise NotImplementedError

    def isDurationEstimated(self):
        # type: () -> bool
        raise NotImplementedError

    def isTimeBased(self):
        # type: () -> bool
        raise NotImplementedError

    def toString(self):
        # type: () -> String
        raise NotImplementedError


class Temporal(TemporalAccessor):
    def minus(self, amount, unit=None):
        # type: (long, Optional[TemporalUnit]) -> Temporal
        pass

    def plus(self, amount, unit=None):
        # type: (long, Optional[TemporalUnit]) -> Temporal
        pass

    def until(self, endExclusive, unit):
        # type: (Temporal, TemporalUnit) -> long
        raise NotImplementedError


class ChronoUnit(Enum, TemporalUnit):
    CENTURIES = None  # type: ChronoUnit
    DAYS = None  # type: ChronoUnit
    DECADES = None  # type: ChronoUnit
    ERAS = None  # type: ChronoUnit
    FOREVER = None  # type: ChronoUnit
    MICROS = None  # type: ChronoUnit
    MILLENNIA = None  # type: ChronoUnit
    MILLIS = None  # type: ChronoUnit
    MINUTES = None  # type: ChronoUnit
    MONTHS = None  # type: ChronoUnit
    NANOS = None  # type: ChronoUnit
    SECONDS = None  # type: ChronoUnit
    HALF_DAYS = None  # type: ChronoUnit
    HOURS = None  # type: ChronoUnit
    WEEKS = None  # type: ChronoUnit
    YEARS = None  # type: ChronoUnit

    def addTo(self, temporal, amount):
        # type: (Temporal, long) -> Temporal
        pass

    def between(self, temporal1Inclusive, temporal2Exclusive):
        # type: (Temporal, Temporal) -> Temporal
        pass

    def getDuration(self):
        # type: () -> Duration
        pass

    def isDateBased(self):
        # type: () -> bool
        pass

    def isDurationEstimated(self):
        # type: () -> bool
        pass

    def isTimeBased(self):
        # type: () -> bool
        pass

    @staticmethod
    def values():
        # type: () -> List[ChronoUnit]
        pass


class ValueRange(Object):
    def checkValidIntValue(self, value, field):
        # type: (long, TemporalField) -> int
        pass

    def checkValidValue(self, value, field):
        # type: (long, TemporalField) -> long
        pass

    def getLargestMinimum(self):
        # type: () -> long
        pass

    def getMaximum(self):
        # type: () -> long
        pass

    def getMinimum(self):
        # type: () -> long
        pass

    def getSmallestMaximum(self):
        # type: () -> long
        pass

    def isFixed(self):
        # type: () -> bool
        pass

    def isIntValue(self):
        # type: () -> bool
        pass

    def isValidIntValue(self, value):
        # type: (long) -> bool
        pass

    def isValidValue(self, value):
        # type: (long) -> bool
        pass

    @staticmethod
    def of(*args):
        # type: (*Any) -> ValueRange
        pass



from __future__ import print_function

__all__ = [
    "INavUtilities",
    "NavUtilities",
    "PrintUtilities",
    "WindowUtilities",
]

from typing import Any, Dict, Optional, Union

from com.inductiveautomation.factorypmi.application import FPMIWindow
from java.awt import Component
from java.awt.image import BufferedImage
from java.lang import Object, String
from java.util import EventObject
from org.python.core import PyObject


class INavUtilities(object):
    """Parent interface to coordinate the functions between NavUtilities
    and NavUtilitiesDispatcher.
    """

    def centerWindow(self, arg):
        # type: (Union[FPMIWindow, String]) -> None
        raise NotImplementedError

    def closeParentWindow(self, event):
        # type: (EventObject) -> None
        raise NotImplementedError

    def closeWindow(self, arg):
        # type: (Union[FPMIWindow, String]) -> None
        raise NotImplementedError

    def getCurrentWindow(self):
        # type: () -> String
        raise NotImplementedError

    def goBack(self):
        # type: () -> PyObject
        raise NotImplementedError

    def goForward(self):
        # type: () -> PyObject
        raise NotImplementedError

    def goHome(self):
        # type: () -> PyObject
        raise NotImplementedError

    def openWindow(self, path, params=None):
        # type: (String, Optional[Dict[String, Any]]) -> PyObject
        raise NotImplementedError

    def openWindowImpl(self, path, params, openAdditional):
        # type: (String, Dict[String, Any], bool) -> PyObject
        raise NotImplementedError

    def openWindowInstance(self, path, params=None):
        # type: (String, Optional[Dict[String, Any]]) -> PyObject
        raise NotImplementedError

    def swapTo(self, name, params):
        # type: (String, Dict[String, Any]) -> PyObject
        raise NotImplementedError

    def swapWindow(self, *args):
        # type: (*Any) -> PyObject
        raise NotImplementedError


class NavUtilities(INavUtilities):
    def centerWindow(self, arg):
        # type: (Union[FPMIWindow, String]) -> None
        pass

    def closeParentWindow(self, event):
        # type: (EventObject) -> None
        pass

    def closeWindow(self, arg):
        # type: (Union[FPMIWindow, String]) -> None
        pass

    def getCurrentWindow(self):
        # type: () -> String
        pass

    def goBack(self):
        # type: () -> PyObject
        pass

    def goForward(self):
        # type: () -> PyObject
        pass

    def goHome(self):
        # type: () -> PyObject
        pass

    def openWindow(self, path, params=None):
        # type: (String, Optional[Dict[String, Any]]) -> PyObject
        pass

    def openWindowImpl(self, path, params, openAdditional):
        # type: (String, Dict[String, Any], bool) -> PyObject
        pass

    def openWindowInstance(self, path, params=None):
        # type: (String, Optional[Dict[String, Any]]) -> PyObject
        pass

    def swapTo(self, name, params):
        # type: (String, Dict[String, Any]) -> PyObject
        pass

    def swapWindow(self, *args):
        # type: (*Any) -> PyObject
        pass


class PrintUtilities(Object):
    def __init__(self, app):
        # type: (Any) -> None
        print(self, app)

    def createImage(self, c):
        # type: (Component) -> BufferedImage
        print(self, c)
        width = height = imageType = 1
        return BufferedImage(width, height, imageType)

    def createPrintJob(self, c):
        # type: (Component) -> PrintUtilities.JythonPrintJob
        pass

    def printToImage(self, c, fileName=None):
        # type: (Component, Optional[str]) -> None
        pass

    class JythonPrintJob(Object):
        def getBottomMargin(self):
            pass

        def getLeftMargin(self):
            pass

        def getOrientation(self):
            pass

        def getPageHeight(self):
            pass

        def getPageWidth(self):
            pass

        def getPrinterName(self):
            pass

        def getRightMargin(self):
            pass

        def getTopMargin(self):
            pass

        def getZoomFactor(self):
            pass

        def isFitToPage(self):
            pass

        def isShowPrintDialog(self):
            pass

        def setBottomMargin(self, bottomMargin):
            pass

        def setFitToPage(self, fitToPage):
            pass

        def setLeftMargin(self, leftMargin):
            pass

        def setMargins(self, m):
            pass

        def setOrientation(self, orientation):
            pass

        def setPageHeight(self, pageHeight):
            pass

        def setPageWidth(self, pageWidth):
            pass

        def setPrinterName(self, printerName):
            pass

        def setRightMargin(self, rightMargin):
            pass

        def setShowPrintDialog(self, showPrintDialog):
            pass

        def setZoomFactor(self, zoomFactor):
            pass


class WindowUtilities(Object):
    """These are the scripting functions mounted at system.gui.*.
    Changes to this class must be made carefully, as some of the true
    implementations actually reside in the subclass,
    WindowUtilitiesForDesktop.
    """

    def confirm(self, *args):
        pass

    def errorBox(self, *args):
        pass

    def inputBox(self, *args):
        pass

    def messageBox(self, *args):
        pass

    def passwordBox(self, *args):
        pass

    def warningBox(self, *args):
        pass



"""Math Functions.

The following functions assist with running statistical analysis.
"""

from __future__ import print_function

__all__ = [
    "geometricMean",
    "kurtosis",
    "max",
    "mean",
    "meanDifference",
    "median",
    "min",
    "mode",
    "normalize",
    "percentile",
    "populationVariance",
    "product",
    "skewness",
    "standardDeviation",
    "sum",
    "sumDifference",
    "sumLog",
    "sumSquares",
    "variance",
]

import __builtin__ as builtins

from typing import List, Union

from org.apache.commons.math3.exception import DimensionMismatchException


def geometricMean(values):
    # type: (List[Union[float, int]]) -> float
    """Calculates the geometric mean.

    Geometric Mean is a type of average which indicates a typical value
    in a set of numbers by using the product of values in the set.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The geometric mean, or NaN if the input was empty or null.
        Because this uses logs to compute the geometric mean, will
        return NaN if any entries are negative.
    """
    print(values)
    return float(43)


def kurtosis(values):
    # type: (List[Union[float, int]]) -> float
    """Calculates the kurtosis of a sequence of values.

    Kurtosis measures if data is peaked or flat relative to normal
    distribution. A set of data with high kurtosis will have distinct
    peaks near the mean, while a set of data with low kurtosis will have
    a flat top near the mean. Uniform distribution is typically a flat
    line.

    Returns NaN (Not a Number) if passed an empty sequence measure of
    whether the data are heavy-tailed or light-tailed of a given
    distribution.

    Args:
        values: A sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The kurtosis, or NaN if the input was empty or null.
        Additionally, returns NaN if the values returned fewer than 4
        values.
    """
    print(values)
    return float(43)


def max(values):
    # type: (List[Union[float, int]]) -> float
    """Given a sequence of values, returns the greatest value in the
    sequence, also known as the "max" value.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The maximum value contained in the values parameter, or NaN if
        the input was empty or null.
    """
    return builtins.max(values)


def mean(values):
    # type: (List[Union[float, int]]) -> float
    """Given a sequence of values, calculates the arithmetic mean
    (average).

    Returns NaN (Not a Number) if passed an empty sequence, a null.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The arithmetic mean, or NaN if the input was empty or None.
    """
    print(values)
    return float(43)


def meanDifference(values1, values2):
    # type: (List[Union[float, int]], List[Union[float, int]]) -> float
    """Given two sequences of values, calculates the mean of the signed
    difference between both sequences.

    In other words, returns the absolute difference between the mean
    values of two different sets of data.

    Args:
        values1: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.
        values2: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The mean difference, or NaN if one of the parameters was empty
        or null.

    Raises:
        DimensionMismatchException: If the two sequences have different
            lengths.
    """
    if len(values1) != len(values2):
        raise DimensionMismatchException()
    return float(43)


def median(values):
    # type: (List[Union[float, int]]) -> float
    """Takes a sequence of values, and returns the median.

    The Median represents the middle value in a set of data.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The median, or NaN if the input was empty or null.
    """
    print(values)
    return float(43)


def min(values):
    # type: (List[Union[float, int]]) -> float
    """Given a Sequence of numerical values, returns the minimum value,
    also known as the "min" value.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The minimum value contained within the values parameter, or NaN
        if the input was empty or null.
    """
    return builtins.min(values)


def mode(values):
    # type: (List[Union[float, int]]) -> List[float]
    """Given a sequence of values, returns the 'mode', or most frequent
    values.

    Returns an empty list if the sequence was empty or None.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.

    Returns:
        A Java Array (functionally similar to a Python List) of floats
        representing the most frequent values in the values parameter.
        If the values parameter was empty, then an empty list will be
        returned instead.
    """
    print(values)
    return [float(43)]


def normalize(values):
    # type: (List[Union[float, int]]) -> List[float]
    """Given a sequence of values, normalizes the values.

    Normalizing data refers to adjusting values measured on different
    scales and brings them into alignment to allow the comparison of
    corresponding normalized values. This creates uniformity of values
    by eliminating the different units of measurement, and to more
    easily compare data from different places

    Returns an empty list if the sequence was empty or None.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.

    Returns:
        A Java Array (functionally similar to a Python List) of floats
        representing normalized input, with a mean of 0 and a standard
        deviation of 1. Returns an empty array if the input was empty or
        None. If the standard deviation is 0, will return an array of
        float NaN (Not a Number).
    """
    print(values)
    return [float(43)]


def percentile(values, percentile):
    # type: (List[Union[float, int]], float) -> float
    """Given a sequence of numerical values, estimates the percentile of
    input.

    The percentile is a value on a scale that represents a percentage
    position in a list of data that can be equal to or below that value:
    i.e., the 25th percentile is a value below which 25% of observable
    data points may be found.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.
        percentile: The percentile to compute. A float greater than 0
            and less than or equal to 100. Will throw an exception if
            the percentile is out of bounds.

    Returns:
        An estimate of the requested percentile of the input, or NaN if
        the input was empty or null.
    """
    print(values, percentile)
    if 0 > percentile > 100:
        raise ValueError("percentile out of bounds.")
    return float(43)


def populationVariance(values):
    # type: (List[Union[float, int]]) -> float
    """Given a sequence of values, returns the Population Variance.

    Population variance calculates how values in a dataset are spread
    out from their average value.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The population variance, or NaN if the input was empty or null.
    """
    print(values)
    return float(43)


def product(values):
    # type: (List[Union[float, int]]) -> float
    """Given a sequence of values, calculates the product of the
    sequence: the result of multiplying of all values in the sequence
    together.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The product of all values in the values parameter, or NaN if the
        input was empty or null.
    """
    print(values)
    return float(43)


def skewness(values):
    # type: (List[Union[float, int]]) -> float
    """Calculates the skewness given a sequence of values.

    Skewness is a measure of the degree of asymmetry of a distribution
    of the mean. If skewed to the left, the distribution has a long tail
    in the negative direction. If skewed to the right, the tail will be
    skewed in the positive direction.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The skewness of the values parameter, or NaN if values was empty
        or null.
    """
    print(values)
    return float(43)


def standardDeviation(values):
    # type: (List[Union[float, int]]) -> float
    """Given a Sequence of numerical values, calculates the standard
    deviation.

    Standard deviation is a calculated number for how close, or how far
    the values of that dataset are in relation to the mean.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The standard deviation of the values parameter, or NaN if the
        values was empty or null.
    """
    print(values)
    return float(43)


def sum(values):
    # type: (List[Union[float, int]]) -> float
    """Given a sequence of values, calculates the sum of all values.

    The sum is the number returned by addition.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The sum of all values in the values parameter, or NaN if values
        was empty or null.
    """
    return builtins.sum(values)


def sumDifference(values1, values2):
    # type: (List[Union[float, int]], List[Union[float, int]]) -> float
    """Given two sequences of values, calculates the sum of the signed
    difference between both sequences.

    In other words, the sum and difference between two sets of numbers.

    Args:
        values1: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.
        values2: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The sum difference, or NaN if one of the parameters was empty or
        null.

    Raises:
        DimensionMismatchException: If the two sequences have different
            lengths.
    """
    if len(values1) != len(values2):
        raise DimensionMismatchException()
    return float(43)


def sumLog(values):
    # type: (List[Union[float, int]]) -> float
    """Given a sequence of values, calculates the sum of the natural
    logs.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The sum of the natural logs of the input values, or NaN if the
        input was empty, None, or contains negative numbers.
    """
    print(values)
    return float(43)


def sumSquares(values):
    # type: (List[Union[float, int]]) -> float
    """Given a sequence of values, calculates the sum of the squares of
    all values.

    Sum squares measures how far individual values are from the mean by
    calculating how much variation there is in a set of values.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The sum of all squares of the values parameter, or NaN if the
        input was empty or null.
    """
    return builtins.sum(value**2 for value in values)


def variance(values):
    # type: (List[Union[float, int]]) -> float
    """Given a sequence of values, calculates the variance of all
    values.

    Variance measures how far values in a set are spread out from their
    mean value.

    Returns NaN (Not a Number) if passed an empty sequence.

    Args:
        values: A Sequence of numerical values. Accepts both Integers
            and Floats. The sequence may not contain None type values.
            However, passing a None type object instead of a Sequence of
            numerical values will return NaN.

    Returns:
        The variance of all values in the values parameter, or NaN if
        the input was empty or null.
    """
    print(values)
    return float(43)



"""thecesrom.dev utils decorators."""


class classproperty(property):  # pylint: disable=invalid-name
    def __get__(self, cls, owner):  # type: ignore
        return classmethod(self.fget).__get__(None, owner)()



from typing import Optional

from com.inductiveautomation.ignition.common.gson import Gson, JsonObject
from com.inductiveautomation.ignition.common.jsonschema import JsonSchema
from com.inductiveautomation.ignition.common.util import LoggerEx
from java.lang import Object, String
from java.util.function import Consumer


class PerspectiveModule(Object):
    LOG_PREFIX = None  # type: String
    META_SCHEMA = None  # type: JsonSchema
    MODULE_ID = None  # type: String
    SESSION_PROPS_SCHEMA = None  # type: JsonSchema
    VIEW_SCHEMA = None  # type: JsonSchema

    @staticmethod
    def createPerspectiveCompatibleGson(customize=None):
        # type: (Optional[Consumer]) -> Gson
        pass

    @staticmethod
    def defaultMetaProps():
        # type: () -> JsonObject
        pass

    @staticmethod
    def defaultSessionProps():
        # type: () -> JsonObject
        pass

    @staticmethod
    def defaultViewProps():
        # type: () -> JsonObject
        pass

    @staticmethod
    def getLogger(name):
        # type: (String) -> LoggerEx
        pass
